/* ------------------------------------------------------------------------- */
/*   i2c-riva.c I2C adapter driver for NVIDIA display adapters		     */
/* ------------------------------------------------------------------------- */
/*   Copyright (C) 2000 Ferenc Bakonyi <fero@drama.obuda.kando.hu>
 *   Based on various sources by Simon G. Vogl
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.		     */
/* ------------------------------------------------------------------------- */

/* Module versioning */
#include <linux/config.h>
#if defined (CONFIG_MODVERSIONS) && !defined (MODVERSIONS)
#define MODVERSIONS
#endif
#include <linux/version.h>
#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
/* end module versioning */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/smp_lock.h>

#include "i2c-riva.h"
#include "rivatv.h"

/* ------------------------------------------------------------------------- *
 *
 * I2C bus
 *
 * ------------------------------------------------------------------------- */

#define BUS0_USED 0x01
#define BUS1_USED 0x02
#define BUS2_USED 0x04

static int __init rivatv_setup_i2c (struct rivatv_info *info,
				    const struct i2c_riva_initdata *i2c_init)
{
	int err;
	struct i2c_riva_info *i2c = info->i2c;
	struct i2c_algo_bit_data *algo = i2c_init->algo;
	struct i2c_adapter *bus0 = i2c_init->bus0;
	struct i2c_adapter *bus1 = i2c_init->bus1;
	struct i2c_adapter *bus2 = i2c_init->bus2;

	// No I2C bus used so far
	i2c->bus_used = 0;

	switch (info->chip.realarch) {
	case NV_ARCH_17:
	case NV_ARCH_25: // Bus 2 is used by GeForce4 & 4MX
		memcpy (&i2c->i2c_bus2, bus2, sizeof (i2c->i2c_bus2));
		memcpy (&i2c->bit_riva_data2, algo, sizeof (i2c->bit_riva_data2));
		i2c->bus_data2.Read_Reg = 0x50;
		i2c->bus_data2.Write_Reg = 0x51;
		i2c->bus_data2.IO_Addr = info->chip.PCIO;
		i2c->bus_data2.chip = &info->chip;
		i2c->bit_riva_data2.data = &i2c->bus_data2;
		i2c->i2c_bus2.algo_data = &i2c->bit_riva_data2;
		i2c->i2c_bus2.data = i2c;
		err = i2c_init->bus_init (i2c->bit_riva_data2.data);
		if (!err)
			err = i2c_bit_add_bus (&i2c->i2c_bus2);
		if (err)
			return err;
		i2c->bus_used |= BUS2_USED;
		// fall through

	case NV_ARCH_04:
	case NV_ARCH_05:
	case NV_ARCH_10:
	case NV_ARCH_11:
	case NV_ARCH_15:
	case NV_ARCH_20: // Bus 1 is used by Riva TNT and up
		memcpy (&i2c->i2c_bus1, bus1, sizeof (i2c->i2c_bus1));
		memcpy (&i2c->bit_riva_data1, algo, sizeof (i2c->bit_riva_data1));
		i2c->bus_data1.Read_Reg = 0x36;
		i2c->bus_data1.Write_Reg = 0x37;
		i2c->bus_data1.IO_Addr = info->chip.PCIO;
		i2c->bus_data1.chip = &info->chip;
		i2c->bit_riva_data1.data = &i2c->bus_data1;
		i2c->i2c_bus1.algo_data = &i2c->bit_riva_data1;
		i2c->i2c_bus1.data = i2c;
		err = i2c_init->bus_init (i2c->bit_riva_data1.data);
		if (!err)
			err = i2c_bit_add_bus (&i2c->i2c_bus1);
		if (err) {
			if (i2c->bus_used & BUS2_USED) i2c_bit_del_bus (&i2c->i2c_bus2);
			i2c->bus_used = 0;
			return err;
		}
		i2c->bus_used |= BUS1_USED;
		// fall through

	case NV_ARCH_03: // Bus 0 is used by all architectures from Riva 128 up.
		memcpy (&i2c->i2c_bus0, bus0, sizeof (i2c->i2c_bus0));
		memcpy (&i2c->bit_riva_data0, algo, sizeof (i2c->bit_riva_data0));
		i2c->bus_data0.Read_Reg = 0x3e;
		i2c->bus_data0.Write_Reg = 0x3f;
		i2c->bus_data0.IO_Addr = info->chip.PCIO;
		i2c->bus_data0.chip = &info->chip;
		i2c->bit_riva_data0.data = &i2c->bus_data0;
		i2c->i2c_bus0.algo_data = &i2c->bit_riva_data0;
		i2c->i2c_bus0.data = i2c;
		err = i2c_init->bus_init (i2c->bit_riva_data0.data);
		if (err)
			return err;
		err = i2c_bit_add_bus (&i2c->i2c_bus0);
		if (err) {
			if (i2c->bus_used & BUS2_USED) i2c_bit_del_bus (&i2c->i2c_bus2);
			if (i2c->bus_used & BUS1_USED) i2c_bit_del_bus (&i2c->i2c_bus1);
			i2c->bus_used = 0;
			return err;
		}
		i2c->bus_used |= BUS0_USED;
		break;

	default:
		PRINTK_ERR ("init I2C failed: unknown architecture\n");
		break;
	}
	return 0;
}

static void __exit rivatv_release_i2c (struct rivatv_info *info)
{
	if (info->i2c->bus_used & BUS0_USED) i2c_bit_del_bus (&info->i2c->i2c_bus0);
	if (info->i2c->bus_used & BUS1_USED) i2c_bit_del_bus (&info->i2c->i2c_bus1);
	if (info->i2c->bus_used & BUS2_USED) i2c_bit_del_bus (&info->i2c->i2c_bus2);
	info->i2c->bus_used = 0;
}

static int __init rivatv_register_i2c (const struct i2c_riva_initdata *driver_init)
{
	int err = -ENOMEM;
	struct rivatv_info *info = riva_board;
	struct i2c_riva_info *i2c;

	if (info == NULL)
		return -ENODEV;

	i2c = kmalloc (sizeof (struct i2c_riva_info), GFP_KERNEL);
	if (!i2c)
		goto err_out;
	memset (i2c, 0, sizeof (struct i2c_riva_info));

	info->i2c = i2c;
	err = rivatv_setup_i2c (info, driver_init);
	if (err)
		goto err_out_kfree;

	return 0;

err_out_kfree:
	info->i2c = NULL;
	kfree (i2c);
err_out:
	return err;
}

static void __exit rivatv_unregister_i2c (void)
{
	struct rivatv_info *info = riva_board;
	struct i2c_riva_info *i2c;
	
	if (info == NULL)
		return;
	i2c = info->i2c;
	rivatv_release_i2c (info);
	info->i2c = NULL;
	kfree (i2c);
}

/*---------------------------------------------------------------------- 
 *
 * I2C adapter driver
 *
 *----------------------------------------------------------------------*/

static void bit_riva_setscl (void *bus, int state);
static void bit_riva_setsda (void *bus, int state);
static int  bit_riva_getscl (void *bus);
static int  bit_riva_getsda (void *bus);
static int  bit_riva_reg (struct i2c_client *client);
static int  bit_riva_unreg (struct i2c_client *client);
static void bit_riva_inc_use (struct i2c_adapter *adap);
static void bit_riva_dec_use (struct i2c_adapter *adap);

static spinlock_t riva_reg_lock = SPIN_LOCK_UNLOCKED;

/* Riva I2C bus manipulation functions and data */
static struct __initdata i2c_algo_bit_data riva_bit_data = {
	data:		NULL,	/* struct riva_i2c_bus * */
	setsda:		bit_riva_setsda,
	setscl:		bit_riva_setscl,
	getsda:		bit_riva_getsda,
	getscl:		bit_riva_getscl,
	udelay:		5,	/* Waits, Timeout: guessed values */
	mdelay:		5,
	timeout:	10,
};

/* For registering I2C adapter driver */
static struct __initdata i2c_adapter i2c_riva_bus0 = {
	name:			"NVIDIA display adapter bus 0",
	id:			I2C_HW_B_RIVA,
	inc_use:		bit_riva_inc_use,
	dec_use:		bit_riva_dec_use,
	client_register:	bit_riva_reg,
	client_unregister:	bit_riva_unreg,
	data:			NULL,	/* struct i2c_riva_info * */
};

static struct __initdata i2c_adapter i2c_riva_bus1 = {
	name:			"NVIDIA display adapter bus 1",
	id:			I2C_HW_B_RIVA,
	inc_use:		bit_riva_inc_use,
	dec_use:		bit_riva_dec_use,
	client_register:	bit_riva_reg,
	client_unregister:	bit_riva_unreg,
	data:			NULL,	/* struct i2c_riva_info * */
};

static struct __initdata i2c_adapter i2c_riva_bus2 = {
	name:			"NVIDIA display adapter bus 2",
	id:			I2C_HW_B_RIVA,
	inc_use:		bit_riva_inc_use,
	dec_use:		bit_riva_dec_use,
	client_register:	bit_riva_reg,
	client_unregister:	bit_riva_unreg,
	data:			NULL,	/* struct i2c_riva_info * */
};

/* Most likely, future nVidia chips will have no I2C support anymore,
   due to Philips suing companies for using their invention.
   Don't you just looooooove lawsuits! *SIGH* */

/*---------------------------------------------------------------------- 
 *
 * Low level hardware manipulation functions
 *
 *----------------------------------------------------------------------*/


/* Possibly: #define DUAL_HEAD_MASK (~0x00000110) */
#define DUAL_HEAD_MASK (~0x00000010)
#define SAVE_DUAL_HEAD_STATE(chip, head)		   \
	(head) = VID_RD32 ((chip)->PMC, 0x600860);	   \
	VID_AND32 ((chip)->PMC, 0x600860, DUAL_HEAD_MASK);
#define RESTORE_DUAL_HEAD_STATE(chip, head)		   \
	VID_WR32 ((chip)->PMC, 0x600860, (head));

/* Locking */
#define LOCK_I2C_RIVA(chip, flags)			  \
	spin_lock_irqsave (&riva_reg_lock, (flags));	  \
	lock_kernel ();					  \
	VID_WR32 ((chip)->PMC, 0x140, 0);
#define UNLOCK_I2C_RIVA(chip, flags)			  \
	VID_WR32 ((chip)->PMC, 0x140, 1);		  \
	unlock_kernel ();				  \
	spin_unlock_irqrestore (&riva_reg_lock, (flags));


/* Set the clock line on the I2C bus */
static void bit_riva_setscl (void *bus, int state)
{
	struct riva_i2c_bus *rb = (struct riva_i2c_bus *) bus;
	unsigned char data;
	unsigned long flags;
	u32 head;

	LOCK_I2C_RIVA (rb->chip, flags);
	SAVE_DUAL_HEAD_STATE (rb->chip, head);

	rb->chip->lock (rb->chip, 0);
	rb->IO_Addr[0x3d4] = rb->Write_Reg;
	data = rb->IO_Addr[0x3d5] & 0xf0;
	if (state)
		data |= 0x20;
	else
		data &= ~0x20;
	rb->IO_Addr[0x3d4] = rb->Write_Reg;
	rb->IO_Addr[0x3d5] = data | 0x01;

	RESTORE_DUAL_HEAD_STATE (rb->chip, head);
	UNLOCK_I2C_RIVA (rb->chip, flags);
}

/* Set the data line on the I2C bus */
static void bit_riva_setsda (void *bus, int state)
{
	struct riva_i2c_bus *rb = (struct riva_i2c_bus *) bus; 
	unsigned char data;
	unsigned long flags;
	u32 head;

	LOCK_I2C_RIVA (rb->chip, flags);
	SAVE_DUAL_HEAD_STATE (rb->chip, head);

	rb->chip->lock (rb->chip, 0);
	rb->IO_Addr[0x3d4] = rb->Write_Reg;
	data = rb->IO_Addr[0x3d5] & 0xf0;
	if (state)
		data |= 0x10;
	else
		data &= ~0x10;
	rb->IO_Addr[0x3d4] = rb->Write_Reg;
	rb->IO_Addr[0x3d5] = data | 0x01;

	RESTORE_DUAL_HEAD_STATE (rb->chip, head);
	UNLOCK_I2C_RIVA (rb->chip, flags);
}

/* Read the clock line on the I2C bus */
static int bit_riva_getscl (void *bus)
{
	struct riva_i2c_bus *rb = (struct riva_i2c_bus *) bus; 
	int data = 0;
	unsigned long flags;
	u32 head;

	LOCK_I2C_RIVA (rb->chip, flags);
	SAVE_DUAL_HEAD_STATE (rb->chip, head);

	rb->chip->lock (rb->chip, 0);
	rb->IO_Addr[0x3d4] = rb->Read_Reg;
	if (rb->IO_Addr[0x3d5] & 0x04)
		data = 1;

	RESTORE_DUAL_HEAD_STATE (rb->chip, head);
	UNLOCK_I2C_RIVA (rb->chip, flags);
	return data;
}

/* Read the data line on the I2C bus */
static int bit_riva_getsda (void *bus)
{
	struct riva_i2c_bus *rb = (struct riva_i2c_bus *) bus; 
	int data = 0;
	unsigned long flags;
	u32 head;

	LOCK_I2C_RIVA (rb->chip, flags);
	SAVE_DUAL_HEAD_STATE (rb->chip, head);

	rb->chip->lock (rb->chip, 0);
	rb->IO_Addr[0x3d4] = rb->Read_Reg;
	if (rb->IO_Addr[0x3d5] & 0x08)
		data = 1;

	RESTORE_DUAL_HEAD_STATE (rb->chip, head);
	UNLOCK_I2C_RIVA (rb->chip, flags);
	return data;
}

/*---------------------------------------------------------------------- 
 *
 * I2C adapter driver interface functions
 *
 *----------------------------------------------------------------------*/

/* Reset the I2C bus lines once */
static int __init bit_riva_init (struct riva_i2c_bus *bus)
{
	bit_riva_setscl (bus, 1);
	bit_riva_setsda (bus, 1);
	return 0;
}

/* This is the top of the thingy */
static int bit_riva_reg (struct i2c_client *client)
{
	struct i2c_riva_info *i2c = (struct i2c_riva_info *) client->adapter->data;
	int id = client->driver->id;

	DPRINTK2 ("chip registration requested: %d (%s)\n", id, client->name);

	if (i2c->video_decoder == NULL)
		if (id == I2C_DRIVERID_SAA7111A ||
		    id == I2C_DRIVERID_SAA7113 ||
		    id == I2C_DRIVERID_SAA7108 ||
		    id == I2C_DRIVERID_VPX32XX ||
		    id == I2C_DRIVERID_TW98) {
			i2c->video_decoder = client;
			PRINTK_INFO ("video decoder chip registered\n");
			return 0;
		}
	if (i2c->tuner == NULL)
		if (id == I2C_DRIVERID_TUNER) {
			i2c->tuner = client;
			PRINTK_INFO ("video tuner chip registered\n");
			return 0;
		}
	if (i2c->audio_processor == NULL)
		/* I2C_DRIVERID_TDA8425 */
		if (id == I2C_DRIVERID_TVAUDIO && client->addr == 0x41) {
			i2c->audio_processor = client;
			PRINTK_INFO ("audio processor chip registered\n");
			return 0;
		}
	if (i2c->audio_decoder == NULL)
		/* I2C_DRIVERID_TDA9850 */
		if (id == I2C_DRIVERID_TVAUDIO && client->addr == 0x5a) {
			i2c->audio_decoder = client;
			PRINTK_INFO ("audio decoder chip registered\n");
			return 0;
		}
	return -ENODEV;
}

static int bit_riva_unreg (struct i2c_client *client)
{
	struct i2c_riva_info *i2c = (struct i2c_riva_info *) client->adapter->data;
	int id = client->driver->id;

	if (i2c->video_decoder != NULL)
		if (id == I2C_DRIVERID_SAA7111A ||
		    id == I2C_DRIVERID_SAA7113 ||
		    id == I2C_DRIVERID_SAA7108 ||
		    id == I2C_DRIVERID_VPX32XX ||
		    id == I2C_DRIVERID_TW98) {
			i2c->video_decoder = NULL;
			PRINTK_INFO ("video decoder chip unregistered\n");
		}
	if (i2c->tuner != NULL)
		if (id == I2C_DRIVERID_TUNER) {
			i2c->tuner = NULL;
			PRINTK_INFO ("video tuner chip unregistered\n");
		}
	if (i2c->audio_processor != NULL)
		if (id == I2C_DRIVERID_TDA8425) {
			i2c->audio_processor = NULL;
			PRINTK_INFO ("audio processor chip unregistered\n");
		}
	if (i2c->audio_decoder != NULL)
		if (id == I2C_DRIVERID_TDA9850) {
			i2c->audio_decoder = NULL;
			PRINTK_INFO ("audio decoder chip unregistered\n");
		}
	return 0;
}

static void bit_riva_inc_use (struct i2c_adapter *adap)
{
	MOD_INC_USE_COUNT;
}

static void bit_riva_dec_use (struct i2c_adapter *adap)
{
	MOD_DEC_USE_COUNT;
}


/*---------------------------------------------------------------------- 
 *
 * Modularization
 *
 *----------------------------------------------------------------------*/

static struct __initdata i2c_riva_initdata driver_init = {
	algo:	  &riva_bit_data,
	bus0:	  &i2c_riva_bus0,
	bus1:	  &i2c_riva_bus1,
	bus2:	  &i2c_riva_bus2,
	bus_init: bit_riva_init
};

int __init init_i2c_riva (void)
{
	PRINTK_INFO ("I2C adapter driver for NVIDIA cards\n");
	return rivatv_register_i2c (&driver_init);
}

#ifdef MODULE
void __exit cleanup_i2c_riva (void) 
{
	rivatv_unregister_i2c ();
}
#endif /* MODULE */
