/* ------------------------------------------------------------------------- */
/*   rivatv-cards.c nVidia based graphics boards list             	     */
/* ------------------------------------------------------------------------- */
/*   Copyright (C) 2002 Stefan Jahn <stefan@lkcc.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.		     */
/* ------------------------------------------------------------------------- */

#include <linux/config.h>
#include <linux/types.h>
#include <linux/init.h>
#include <linux/i2c.h>
#include <linux/pci.h>

#include <tuner.h>

#include "rivatv.h"

struct __initdata rivatv_card rivatv_cards[RIVATV_MAX + 1] = {

	/* PCI_VENDOR_ID_??? -> ASUS or 3D Vision */ 
	{ PCI_VENDOR_ID_NVIDIA_SGS, PCI_DEVICE_ID_NVIDIA_SGS_RIVA128,
	  0x2A15,
	  0x54A3, "Asus V3000 TV", 
	  I2C_DRIVERID_SAA7111A, "saa7111a", { 3, 4, -1 }, -1, -1 },

	/* ASUS */
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_TNT,
	  PCI_VENDOR_ID_ASUSTEK,
	  0x0200, "Asus V3400 TV",
	  I2C_DRIVERID_SAA7111A, "saa7111a", { 3, 4, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_TNT2,
	  PCI_VENDOR_ID_ASUSTEK,
	  0x0200, "Asus V3800 TVR (TNT2)",
	  I2C_DRIVERID_SAA7113, "saa7113h", { 3, 6, 1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_UTNT2,
	  PCI_VENDOR_ID_ASUSTEK,
	  0x0200, "Asus V3800 TVR (Ultra TNT2)",
	  I2C_DRIVERID_SAA7113, "saa7113h", { 3, 6, 1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_TNT2,
	  PCI_VENDOR_ID_ASUSTEK,
	  0x4000, "Asus V3800Pro Deluxe",
	  I2C_DRIVERID_SAA7113, "saa7113h", { 3, 6, 1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_TNT2,
	  PCI_VENDOR_ID_ASUSTEK,
	  0x4017, "Asus V3800 Deluxe",
	  I2C_DRIVERID_SAA7113, "saa7113h", { 3, 6, 1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE_SDR,
	  PCI_VENDOR_ID_ASUSTEK,
	  0x4008, "Asus V6600 Deluxe",
	  I2C_DRIVERID_SAA7113, "saa7113h", { 3, 6, 1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE_DDR,
	  PCI_VENDOR_ID_ASUSTEK,
	  0x400A, "Asus V6800 TVR",
	  I2C_DRIVERID_SAA7113, "saa7113h", { 3, 6, 1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE2_MX,
	  PCI_VENDOR_ID_ASUSTEK,
	  0x4021, "Asus V7100 Combo Deluxe",
	  I2C_DRIVERID_SAA7113, "saa7113h", { 3, 6, 1 }, TUNER_PHILIPS_FQ1216ME, 1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE2_GTS,
	  PCI_VENDOR_ID_ASUSTEK,
	  0x4016, "Asus V7700 Deluxe",
	  I2C_DRIVERID_SAA7113, "saa7113h", { 3, 6, 1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE_SDR,
	  PCI_VENDOR_ID_ASUSTEK,
	  0x4026, "Asus V7700 Deluxe (rev.2 ???)",
	  I2C_DRIVERID_SAA7113, "saa7113h", { 3, 6, 1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE2_GTS2,
	  PCI_VENDOR_ID_ASUSTEK,
	  0x4060, "Asus V7700 TI Deluxe",
	  I2C_DRIVERID_SAA7113, "saa7113h", { 3, 6, 1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE_SDR,
	  PCI_VENDOR_ID_ASUSTEK,
	  0x402F, "Asus V8200 Deluxe",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, 0 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE_SDR,
	  PCI_VENDOR_ID_ASUSTEK,
	  0x4057, "Asus V8200/T2 Deluxe",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, 0 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE_SDR,
	  PCI_VENDOR_ID_ASUSTEK,
	  0x405B, "Asus V8200/T5 Deluxe",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, 0 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4400,
	  PCI_VENDOR_ID_ASUSTEK,
	  0x802F, "Asus V8440 Deluxe",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, 0 }, -1, -1 },

	/* ELSA */
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE_SDR,
	  PCI_VENDOR_ID_ELSA,
	  0x0C10, "Elsa Victory Erazor",
	  I2C_DRIVERID_VPX32XX, "vpx32xx", { 1, 7, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_TNT2,
	  PCI_VENDOR_ID_ELSA,
	  0x0C29, "Elsa Erazor III Video",
	  I2C_DRIVERID_VPX32XX, "vpx32xx", { 1, 7, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_TNT2,
	  PCI_VENDOR_ID_ELSA,
	  0x0C32, "Elsa Erazor III Pro Video",
	  I2C_DRIVERID_VPX32XX, "vpx32xx", { 1, 7, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE2_MX,
	  PCI_VENDOR_ID_ELSA,
	  0x0C60, "Elsa Gladiac Geforce2 MX VIVO",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },

        /* Gainward */
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE2_MX,
	  PCI_VENDOR_ID_GAINWARD,
	  0x0603, "Gainward CARDEXpert GeForce2 MX 400 VIVO",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE2_GTS2,
	  PCI_VENDOR_ID_GAINWARD,
	  0x03EA, "Gainward CARDEXpert GeForce2 Ti 500",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE2_GTS2,
	  PCI_VENDOR_ID_GAINWARD,
	  0x0BCC, "Gainward GeForce2 Ti 500 XP VIVO Golden Sample",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4400,
	  PCI_VENDOR_ID_GAINWARD,
	  0x1460, "Gainward CARDEXpert GeForce4 Ti 4400/Ultra",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE_SDR,
	  PCI_VENDOR_ID_GAINWARD,
	  0x1428, "Gainward CARDEXpert GeForce4 Ti 4600",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4200,
	  PCI_VENDOR_ID_GAINWARD,
	  0x1603, "Gainward GF4 PowerPack! Ultra/650XP VIVO",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },

	/* MSI */
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE_SDR,
	  PCI_VENDOR_ID_MSI,
	  0x3601, "MSI G2Ti PRO-VT(G)",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE2_MX,
	  PCI_VENDOR_ID_MSI,
	  0x8833, "MSI MX400 Pro-VT32S",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE3_1,
	  PCI_VENDOR_ID_MSI,
	  0x8838, "MSI G3Ti200 PRO-VT(G)",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE3_1,
	  PCI_VENDOR_ID_MSI,
	  0x8503, "MSI G3Ti200 Pro Video",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4400,
	  PCI_VENDOR_ID_MSI,
	  0x8711, "MSI G4Ti4400-VTD",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4600,
	  PCI_VENDOR_ID_MSI,
	  0x8725, "MSI G4Ti4600-VTD",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },

	/* AOpen */
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE2_MX,
	  PCI_VENDOR_ID_AOPEN,
	  0x003D, "AOpen MX400-A32",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, 1 }, TUNER_PHILIPS_PAL, 1 },

	/* Leadtek */
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE_SDR,
	  PCI_VENDOR_ID_LEADTEK,
	  0x287C, "Leadtek Winfast A170 Pro TD",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4200,
	  PCI_VENDOR_ID_LEADTEK,
	  0x289C, "Leadtek Winfast A250 TD GF4 Ti 4200 ViVo",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },

	/* Prolink */
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4MX460,
	  PCI_VENDOR_ID_PROLINK,
	  0x1101, "Prolink Pixelview Geforce4 MX460",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },

	/* VisionTek */
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4600,
	  PCI_VENDOR_ID_VISIONTEK,
	  0x0037, "VisionTek Xtasy GeForce4 Ti 4600",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },

	/* unset vendors possibly: Visiontek, Prolink, AOpen, PowerColor */
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE2_MX,
	  0x0000,
	  0x0000, "VisionTek Xtasy Everything Personal Cinema",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_UVTNT2,
	  0x0000,
	  0x0000, "PowerColor CM64A-TNT2 M64",
	  I2C_DRIVERID_TW98, "tw98", { 2, 5, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4200,
	  0x0000,
	  0x0000, "Prolink Pixelview GeForce4 Ti4200",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4200,
	  0x0000,
	  0x0000, "AOpen Aelous GeForce4 Ti4200",
	  I2C_DRIVERID_SAA7108, "saa7108e", { 0, 6, -1 }, -1, -1 },
	
	/* end of list */
	{ PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, 0x0000, "", 0x0000, "", { -1, -1, -1 }, -1, -1 }
};

struct __initdata rivatv_vendor rivatv_vendors[] = {
	{ 0x2A15,                  "Asus (?)" },
	{ PCI_VENDOR_ID_ASUSTEK,   "Asus" },
	{ PCI_VENDOR_ID_ELSA,      "ELSA" },
	{ PCI_VENDOR_ID_GAINWARD,  "Gainward" },
	{ PCI_VENDOR_ID_MSI,       "MSI" },
	{ PCI_VENDOR_ID_AOPEN,     "AOpen" },
	{ PCI_VENDOR_ID_LEADTEK,   "Leadtek" },
	{ PCI_VENDOR_ID_PROLINK,   "Prolink" },
	{ PCI_VENDOR_ID_VISIONTEK, "VisionTek" },
	{ PCI_ANY_ID, "" }
};
