#ifndef __RIVATV_H
#define __RIVATV_H

#include <linux/config.h>
#include <linux/types.h>
#include <linux/tqueue.h>
#include <linux/videodev.h>
#include <linux/video_decoder.h>

/* Debugging definitions */

#define RIVATV_PFX "rivatv: "

#define PRINTK(format, args...)			       \
	printk (RIVATV_PFX format, ## args)
#define PRINTK_ERR(format, args...)		       \
	printk (RIVATV_PFX format, ## args)
#define PRINTK_INFO(format, args...)		       \
	printk (RIVATV_PFX format, ## args)
#define DPRINTK(format, args...) if (debug >= 1)       \
	printk (RIVATV_PFX format, ## args)
#define DPRINTK2(format, args...) if (debug >= 2)      \
	printk (RIVATV_PFX format, ## args)

/* Extra PCI id's. */
#ifndef PCI_DEVICE_ID_NVIDIA_GEFORCE3
#define PCI_DEVICE_ID_NVIDIA_GEFORCE3		0x200
#endif
#ifndef PCI_DEVICE_ID_NVIDIA_GEFORCE3_1
#define PCI_DEVICE_ID_NVIDIA_GEFORCE3_1		0x201
#endif
#ifndef PCI_DEVICE_ID_NVIDIA_GEFORCE3_2
#define PCI_DEVICE_ID_NVIDIA_GEFORCE3_2		0x202
#endif
#ifndef PCI_DEVICE_ID_NVIDIA_GEFORCE4MX460
#define PCI_DEVICE_ID_NVIDIA_GEFORCE4MX460	0x170
#endif
#ifndef PCI_DEVICE_ID_NVIDIA_GEFORCE4MX440
#define PCI_DEVICE_ID_NVIDIA_GEFORCE4MX440	0x171
#endif
#ifndef PCI_DEVICE_ID_NVIDIA_GEFORCE4MX420
#define PCI_DEVICE_ID_NVIDIA_GEFORCE4MX420	0x172
#endif
#ifndef PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4600
#define PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4600	0x250
#endif
#ifndef PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4400
#define PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4400	0x251
#endif
#ifndef PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4200
#define PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4200	0x253
#endif

#ifndef PCI_VENDOR_ID_GAINWARD
#define PCI_VENDOR_ID_GAINWARD	0x10B0
#endif
#ifndef PCI_VENDOR_ID_MSI
#define PCI_VENDOR_ID_MSI	0x1462
#endif
#ifndef PCI_VENDOR_ID_AOPEN
#define PCI_VENDOR_ID_AOPEN	0xA0A0
#endif
#ifndef PCI_VENDOR_ID_PROLINK
#define PCI_VENDOR_ID_PROLINK	0x1554
#endif
#ifndef PCI_VENDOR_ID_VISIONTEK
#define PCI_VENDOR_ID_VISIONTEK	0x1545
#endif

/* Extra I2C ids. */
#ifndef I2C_DRIVERID_SAA7108
#define I2C_DRIVERID_SAA7108	0x46
#endif
#ifndef I2C_DRIVERID_TW98
#define I2C_DRIVERID_TW98	0x47
#endif

/* Quick and dirty hack for extra debugging commands in I2C drivers */

#define READ_REGISTERS		0xBF01
#define WRITE_REGISTERS		0xBF02
#define GET_NR_OF_REGISTERS	0xBF03
#define READ_REGISTER		0xBF04
#define WRITE_REGISTER		0xBF05

/*
 * PCI-Memory IO access macros.
 */
#define VID_WR08(p,i,val)  (((uint8_t *)(p))[(i)]=(val))
#define VID_RD08(p,i)	   (((uint8_t *)(p))[(i)])

#define VID_WR32(p,i,val)  (((uint32_t *)(p))[(i)/4]=(val))
#define VID_RD32(p,i)	   (((uint32_t *)(p))[(i)/4])

#ifdef NO_RMW_CYCLES
/* Can be used to inhibit READ-MODIFY-WRITE cycles. */
#define VID_WR32(p,i,val) \
  ({ __asm__ __volatile__ ("" : : : "memory"); ((uint32_t *)(p))[(i)/4]=(val); })
#define VID_RD32(p,i) \
  ({ __asm__ __volatile__ ("" : : : "memory"); ((uint32_t *)(p))[(i)/4]; })
#endif

#define VID_AND32(p,i,val) VID_WR32(p,i,VID_RD32(p,i)&(val))
#define VID_OR32(p,i,val)  VID_WR32(p,i,VID_RD32(p,i)|(val))
#define VID_XOR32(p,i,val) VID_WR32(p,i,VID_RD32(p,i)^(val))


/*
 * Define supported architectures.
 */
#define NV_ARCH_03  0x03
#define NV_ARCH_04  0x04
#define NV_ARCH_10  0x10
#define NV_ARCH_20  0x20

/*
 * Additional architectures, used for i2c detection.
 */
#define NV_ARCH_05  0x05	// Riva TNT2
#define NV_ARCH_11  0x11	// GeForce 2 MX
#define NV_ARCH_17  0x17	// GeForce 4 MX
#define NV_ARCH_15  0x15	// GeForce 2
#define NV_ARCH_25  0x25	// GeForce 4 Ti


#define RIVATV_MIN_WIDTH  64  /* minimal width */
#define RIVATV_MIN_HEIGHT 32  /* minimal height */
#define RIVATV_MAX_WIDTH  720 /* maximum width */
#define RIVATV_MAX_HEIGHT 576 /* maximum height */

/* Maximum capture queue buffers. */
#define RIVATV_MAX_CAPTURE_BUFFERS 32
/* Default capture queue buffers. */
#define RIVATV_CAPTURE_BUFFERS 2
/* Default capture buffer size. */
#define RIVATV_CAPTURE_BUFSIZE \
  PAGE_ALIGN (RIVATV_MAX_WIDTH * RIVATV_MAX_HEIGHT * 4)
/* Default raw (UYVY) capture buffer size. */
#define RIVATV_RAW_CAPTURE_BUFSIZE \
  PAGE_ALIGN (RIVATV_MAX_WIDTH * RIVATV_MAX_HEIGHT * 2)

/* Size of the nicename char-array. DO NOT WRITE BEYOND END OF BUFFER. */
#define RIVATV_NICENAME_BUFSIZE 51

/* Macro checking if the given video format is supported. */
#define RIVATV_PALETTE_SUPPORTED(pal) \
  ((pal) == VIDEO_PALETTE_RGB24	   || \
   (pal) == VIDEO_PALETTE_RGB32	   || \
   (pal) == VIDEO_PALETTE_RGB555   || \
   (pal) == VIDEO_PALETTE_RGB565   || \
   (pal) == VIDEO_PALETTE_YUV422   || \
   (pal) == VIDEO_PALETTE_YUV411   || \
   (pal) == VIDEO_PALETTE_YUYV	   || \
   (pal) == VIDEO_PALETTE_GREY	   || \
   (pal) == VIDEO_PALETTE_UYVY     || \
   (pal) == VIDEO_PALETTE_YUV420P  || \
   (pal) == VIDEO_PALETTE_YUV410P  || \
   (pal) == VIDEO_PALETTE_YUV411P  || \
   (pal) == VIDEO_PALETTE_YUV422P)

#define VIDEO_INTERLACED     8
#define VIDEO_FLAG_1	     1

#ifndef TUNER_PHILIPS_FQ1216ME
#define TUNER_PHILIPS_FQ1216ME 24
#endif

/* Structure defining a video format. */

struct rivatv_format {
	int norm;      /* one of the VIDEO_MODE_ defines    */
	int width;     /* width in pixels		    */
	int height;    /* height in pixels		    */
	int size;      /* size of requested capture buffer  */
	int format;    /* one of the VIDEO_PALETTE_ defines */
	int overlay;   /* overlay indicator flag	    */
};

/* Structure defining a video port. */

struct rivatv_port {
	int max_width;	/* maximum width in pixels		   */
	int max_height; /* maximum height per field in pixels	   */
	int org_width;	/* decoder pixels in framebuffer	   */
	int org_height; /* decoder lines in framebuffer		   */
	int vld_width;	/* number of valid pixels in a line	   */
	int vld_height; /* valid lines				   */
	int width;	/* size of a video line in pixels	   */
	int height;	/* number of video lines		   */
	int flags;	/* VIDEO_INTERLACED, VIDEO_FLAG_1	   */
	int size;	/* size of maximum capture region in bytes */
	u32 state_buf_notify;
	u32 state_buf_usage;
};

/* Structure defining a video source. */

struct rivatv_source {
	int id;		/* decoder chip id				      */
	int mode[3];	/* input modes: 0 - Composite, 1 - S-Video, 2 - Tuner */
	int formats[8]; /* possible video formats (VIDEO_PALETTE_)	      */
	int format;	/* current video format				      */
};

/* Structure defining a single capture queue buffer. */

struct rivatv_cap_buffer {
	int status;  /* status of a capture queue buffer */

#define RIVATV_UNUSED	1 /* not used			  */
#define RIVATV_GRABBING 2 /* currently under construction */
#define RIVATV_ERROR	4 /* error occurred		  */
#define RIVATV_READY	8 /* buffer is ready		  */

	int width;   /* size of a single line	  */
	int height;  /* number of lines to grab	  */
	int format;  /* VIDEO_PALETTE_XXX define  */
	void *addr;  /* virtual address in kernel */
	ulong size;  /* size of this buffer	  */
};

/* Structure defining the capture queue. */

struct rivatv_cap_queue {
	wait_queue_head_t wait; /* the wait queue	  */
	struct tq_struct task;	/* task queue		  */
	int index[RIVATV_MAX_CAPTURE_BUFFERS]; /* capture buffer indices */
	int head;		/* queue start		  */
	int tail;		/* end of queue		  */
	int fill;		/* number of used entries */
};

/* List of v4l devices supported by this driver. */

struct rivatv_initdata {
	struct video_device *video;
	struct video_device *vbi;
};

/* Chip architecture specific info. */

struct rivatv_chip {

	volatile uint32_t *PMC;	   /* general control			*/
	volatile uint32_t *PME;	   /* multimedia port			*/
	volatile uint32_t *PFB;	   /* framebuffer control		*/
	volatile uint32_t *PVIDEO; /* overlay control			*/
	volatile uint8_t *PCIO;	   /* SVGA (CRTC, ATTR) registers	*/
	volatile uint8_t *PVIO;	   /* SVGA (MISC, GRAPH, SEQ) registers */
	volatile uint32_t *PRAMIN; /* instance memory			*/

	ulong fbsize;		   /* framebuffer size		   */
	int arch;		   /* compatible NV_ARCH_XX define */
	int realarch;		   /* real architecture		   */
	void (* lock) (struct rivatv_chip *, int);
};

/* Card identfication info. */

#define RIVATV_MAX 41 /* Maximum number of supported cards, keep in sync with CARDLIST file. */

struct rivatv_card {
	unsigned short vendor;		    /* PCI vendor id					  */
	unsigned short device;		    /* PCI device id					  */
	unsigned short subsystem_vendor;    /* PCI subsystem vendor id				  */
	unsigned short subsystem_device;    /* PCI subsystem device id				  */
	char name[RIVATV_NICENAME_BUFSIZE]; /* complete name					  */
	int id;				    /* decoder chip id					  */
	char decoder[16];		    /* decoder chip module name				  */
	int mode[3];			    /* input modes: 0 - Composite, 1 - S-Video, 2 - Tuner */
	int tuner_required;		    /* tuner required					  */
	int audio_required;		    /* audio required					  */
	int identified;                     /* has card been identified correctly ?               */
};

struct rivatv_vendor {
	unsigned short subsystem_vendor;    /* PCI subsystem vendor id				  */
	char name[16];			    /* vendor name (company)				  */
};

/* Overlay definitions. */

struct rivatv_overlay {

	struct video_window window; /* X screen definitions	  */
	struct video_buffer buffer; /* overlay window definitions */
	int enable;		    /* overlay enable flag	  */
	int enabled;                /* overall overlay flag       */
	int interrupts;		    /* number of interrupts	  */
	int index;		    /* overlay image index 0/1	  */
	u32 colorkey;		    /* saved colour key		  */
};

/* Overall video device structure. */

struct rivatv_info {
	int interrupt_count;		 /* number of interrupts		       */
#ifdef CONFIG_MTRR
	int MTRR;			 /* flag if MTRR is enabled		       */
	int MTRR_range;			 /* MTRR index				       */
#endif /* CONFIG_MTRR */
	int transfer_active;		 /* set while picture transfer		       */
	int capture_schedule;		 /* number of scheduled bottom halfs	       */
	struct rivatv_format format;	 /*					       */
	ulong vbi_region_size;		 /* size of teletext region		       */
	struct rivatv_source *source;	 /* decoder chip video source		       */
	long last_jiffies;		 /*					       */
	int device_busy;		 /* is video capture device used	       */
	char *capture_buffer;		 /* aligned capture buffer		       */
	ulong capture_buffer_size;	 /* size of each capture buffer		       */
	int capture_buffers;		 /* number of capture buffers		       */
	ulong buffer_size;		 /* size of the raw capture buffer	       */
	char *data;			 /* capture data buffer			       */
	struct rivatv_cap_buffer capbuf[RIVATV_MAX_CAPTURE_BUFFERS];
	struct rivatv_cap_queue capture_queue;
	struct video_picture picture;	 /* video picture values		       */
	int interrupts;			 /* number of picture interrupts	       */
	int capture_started;		 /* is video capture running ?		       */
	struct rivatv_card card;	 /* card info				       */
	struct rivatv_chip chip;	 /* NV architecture structure		       */
	struct i2c_riva_info *i2c;	 /* interface to I2C clients		       */
	struct video_device video;	 /* V4L (capture) driver		       */
	struct video_device vbi;	 /* VBI (teletext) driver		       */
	struct pci_dev *pci;		 /* pointer to board's pci driver	       */
	const char *driver;		 /* textual description of the PCI card	       */
	char nicecardname[RIVATV_NICENAME_BUFSIZE];
					 /* proper name of the card, if detected       */
	unsigned long base0_region_size; /* length of control region in bytes	       */
	unsigned long base1_region_size; /* length of fb region in bytes	       */
	unsigned long base0_region;	 /* physical address of control region	       */
	unsigned long base1_region;	 /* physical address of fb region	       */
	caddr_t video_base;		 /* virtual address of control region	       */
	caddr_t control_base;		 /* virtual address of fb region	       */
	struct rivatv_port port;	 /* video port settings (in framebuffer)       */
	unsigned long picture_base;	 /* direct pointer to video picture	       */
	unsigned long picture_offset;	 /* offset of video picture in frame buffer    */
	spinlock_t video_lock;		 /* video lock				       */
	spinlock_t queue_lock;		 /* capture queue lock			       */
	struct semaphore decoder_lock;	 /* video decoder lock			       */
	struct rivatv_overlay overlay;	 /* overlay settings			       */
	int tuner_required;		 /* flag indicating if tuner chip requested    */
	int tuner_selected;		 /* flag indicating if tuner is selected       */
	unsigned long tuner_frequency;	 /* cuurent tuner frequency		       */
	int tuner_used;			 /* flag indicating if tuner is used	       */
	int audio_required;		 /* flag indicating if audio is requested      */
	struct video_audio audio;	 /* current audio settings		       */ 
	int audio_decoder_used;		 /* flag indicating if audio decoder is used   */
	int audio_processor_used;	 /* flag indicating if audio processor is used */
	int sources;			 /* number of video inputs		       */
	int channel;                     /* currently selected video source            */
};

/* Image conversion definitions. */

typedef void (* line_convert_t)	 (char *src, char *dst, int width);
typedef void (* plane_convert_t) (char *src, char *dst, int width, int height);

struct rivatv_conversion {
	int bpp;	     /* number of bits per pixel of source format      */
	int planar;	     /* is the image format planar (1) or packed (0) ? */
	char *name;	     /* short name				       */
	void *converter[17]; /* image converter for each destination format    */
};

extern struct rivatv_info *riva_board;
extern struct __initdata rivatv_card rivatv_cards[];
extern struct __initdata rivatv_vendor rivatv_vendors[];
extern struct rivatv_conversion rivatv_convert[17];
extern int isMMX;

/* Configurable module parameters. */

extern int capbuffers;
extern int autoload;
extern int debug;
extern int tvbox;
extern int mtrr;
extern int card;

#endif /* __RIVATV_H */
