/* ------------------------------------------------------------------------- */
/*   v4l-riva.c V4L support functions for NVIDIA display adapters	     */
/* ------------------------------------------------------------------------- */
/*   Copyright (C) 2000 Ferenc Bakonyi <fero@drama.obuda.kando.hu>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.		     */
/* ------------------------------------------------------------------------- */

/* Module versioning */
#include <linux/config.h>
#if defined (CONFIG_MODVERSIONS) && !defined (MODVERSIONS)
#define MODVERSIONS
#endif
#include <linux/version.h>
#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
/* End module versioning */

#include <linux/types.h>
#ifdef CONFIG_KMOD
#include <linux/kmod.h>
#endif
#ifdef CONFIG_MTRR
#include <asm/mtrr.h>
#endif
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/wait.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/slab.h>
#include <linux/pci.h>
#include <linux/vmalloc.h>
#include <linux/init.h>
#include <asm/processor.h>

#include "i2c-riva.h"
#include "tuner.h"
#include "rivatv.h"

/*----------------------------------------------------------------------
 *
 * Global data
 *
 *----------------------------------------------------------------------*/

struct rivatv_info *riva_board = NULL;

/* configurable number of capture queue buffers */
int capbuffers = RIVATV_CAPTURE_BUFFERS;
/* configurable boolean indicating whether (!0) or not (0) to
   autoload a decoder chip module */
int autoload = 1;
/* debug level: 0 = silent, 1 = verbose, 2 = very verbose */
int debug = 1;
/* usage of tv-box: 0 = no tv-box, 1 = PAL tv-box, 2 = NTSC tv-box */
int tvbox = 0;
/* card type */
int card = -1;
#ifdef CONFIG_MTRR
/* mtrr usage flag */
int mtrr = 1;
#endif

/* register values for weight register programming */
static unsigned char rivatv_weight_table [] = {
  /* 0x00000 */
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  /* 0x10000 */
  0xF2, 0xFF, 0xFF, 0xFF, 0x4E, 0x00, 0x00, 0x00, 0x4E, 0x00, 
  0x00, 0x00, 0xF2, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00,
  0xF6, 0xFF, 0xFF, 0xFF, 0x32, 0x00, 0x00, 0x00, 0x65, 0x00, 
  0x00, 0x00, 0xEC, 0xFF, 0xFF, 0xFF, 0x07, 0x00, 0x00, 0x00, 
  0xFD, 0xFF, 0xFF, 0xFF, 0x19, 0x00, 0x00, 0x00, 0x7B, 0x00, 
  0x00, 0x00, 0xEA, 0xFF, 0xFF, 0xFF, 0x05, 0x00, 0x00, 0x00,
  0x02, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x87, 0x00, 
  0x00, 0x00, 0xED, 0xFF, 0xFF, 0xFF, 0x06, 0x00, 0x00, 0x00, 
  0x05, 0x00, 0x00, 0x00, 0xF5, 0xFF, 0xFF, 0xFF, 0x8C, 0x00, 
  0x00, 0x00, 0xF5, 0xFF, 0xFF, 0xFF, 0x05, 0x00, 0x00, 0x00,
  0x06, 0x00, 0x00, 0x00, 0xED, 0xFF, 0xFF, 0xFF, 0x87, 0x00, 
  0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 
  0x05, 0x00, 0x00, 0x00, 0xEA, 0xFF, 0xFF, 0xFF, 0x7B, 0x00, 
  0x00, 0x00, 0x19, 0x00, 0x00, 0x00, 0xFD, 0xFF, 0xFF, 0xFF,
  0x07, 0x00, 0x00, 0x00, 0xEC, 0xFF, 0xFF, 0xFF, 0x65, 0x00, 
  0x00, 0x00, 0x32, 0x00, 0x00, 0x00, 0xF6, 0xFF, 0xFF, 0xFF, 
  /* 0x12B85 */
  0xF6, 0xFF, 0xFF, 0xFF, 0x4A, 0x00, 0x00, 0x00, 0x4A, 0x00, 
  0x00, 0x00, 0xF6, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 
  0xF3, 0xFF, 0xFF, 0xFF, 0x3C, 0x00, 0x00, 0x00, 0x56, 0x00, 
  0x00, 0x00, 0xF9, 0xFF, 0xFF, 0xFF, 0x02, 0x00, 0x00, 0x00, 
  0xF5, 0xFF, 0xFF, 0xFF, 0x30, 0x00, 0x00, 0x00, 0x60, 0x00, 
  0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0xFA, 0xFF, 0xFF, 0xFF,
  0xF6, 0xFF, 0xFF, 0xFF, 0x21, 0x00, 0x00, 0x00, 0x65, 0x00, 
  0x00, 0x00, 0x0A, 0x00, 0x00, 0x00, 0xFA, 0xFF, 0xFF, 0xFF, 
  0xF8, 0xFF, 0xFF, 0xFF, 0x14, 0x00, 0x00, 0x00, 0x68, 0x00, 
  0x00, 0x00, 0x14, 0x00, 0x00, 0x00, 0xF8, 0xFF, 0xFF, 0xFF, 
  0xFA, 0xFF, 0xFF, 0xFF, 0x0A, 0x00, 0x00, 0x00, 0x65, 0x00,
  0x00, 0x00, 0x21, 0x00, 0x00, 0x00, 0xF6, 0xFF, 0xFF, 0xFF, 
  0xFA, 0xFF, 0xFF, 0xFF, 0x01, 0x00, 0x00, 0x00, 0x60, 0x00, 
  0x00, 0x00, 0x30, 0x00, 0x00, 0x00, 0xF5, 0xFF, 0xFF, 0xFF, 
  0x02, 0x00, 0x00, 0x00, 0xF9, 0xFF, 0xFF, 0xFF, 0x56, 0x00, 
  0x00, 0x00, 0x3C, 0x00, 0x00, 0x00, 0xF3, 0xFF, 0xFF, 0xFF,
  /* 0x187AE */
  0x05, 0x00, 0x00, 0x00, 0x3B, 0x00, 0x00, 0x00, 0x3B, 0x00, 
  0x00, 0x00, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x04, 0x00, 0x00, 0x00, 0x37, 0x00, 0x00, 0x00, 0x42, 0x00,
  0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0xF7, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0x2F, 0x00, 0x00, 0x00, 0x44, 0x00, 
  0x00, 0x00, 0x12, 0x00, 0x00, 0x00, 0xFC, 0xFF, 0xFF, 0xFF, 
  0xFD, 0xFF, 0xFF, 0xFF, 0x27, 0x00, 0x00, 0x00, 0x48, 0x00, 
  0x00, 0x00, 0x18, 0x00, 0x00, 0x00, 0xFC, 0xFF, 0xFF, 0xFF,
  0xFC, 0xFF, 0xFF, 0xFF, 0x20, 0x00, 0x00, 0x00, 0x48, 0x00, 
  0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0xFC, 0xFF, 0xFF, 0xFF, 
  0xFC, 0xFF, 0xFF, 0xFF, 0x18, 0x00, 0x00, 0x00, 0x48, 0x00, 
  0x00, 0x00, 0x27, 0x00, 0x00, 0x00, 0xFD, 0xFF, 0xFF, 0xFF, 
  0xFC, 0xFF, 0xFF, 0xFF, 0x12, 0x00, 0x00, 0x00, 0x44, 0x00,
  0x00, 0x00, 0x2F, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xF7, 0xFF, 0xFF, 0xFF, 0x0C, 0x00, 0x00, 0x00, 0x42, 0x00, 
  0x00, 0x00, 0x37, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 
  /* 0x28000 */
  0x16, 0x00, 0x00, 0x00, 0x2A, 0x00, 0x00, 0x00, 0x2A, 0x00,
  0x00, 0x00, 0x16, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x12, 0x00, 0x00, 0x00, 0x27, 0x00, 0x00, 0x00, 0x2B, 0x00, 
  0x00, 0x00, 0x18, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00,
  0x0F, 0x00, 0x00, 0x00, 0x25, 0x00, 0x00, 0x00, 0x2C, 0x00, 
  0x00, 0x00, 0x1A, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 
  0x0C, 0x00, 0x00, 0x00, 0x23, 0x00, 0x00, 0x00, 0x2C, 0x00,
  0x00, 0x00, 0x1D, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 
  0x0A, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x2C, 0x00, 
  0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x0A, 0x00, 0x00, 0x00,
  0x08, 0x00, 0x00, 0x00, 0x1D, 0x00, 0x00, 0x00, 0x2C, 0x00, 
  0x00, 0x00, 0x23, 0x00, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 
  0x06, 0x00, 0x00, 0x00, 0x1A, 0x00, 0x00, 0x00, 0x2C, 0x00,
  0x00, 0x00, 0x25, 0x00, 0x00, 0x00, 0x0F, 0x00, 0x00, 0x00, 
  0x04, 0x00, 0x00, 0x00, 0x18, 0x00, 0x00, 0x00, 0x2B, 0x00, 
  0x00, 0x00, 0x27, 0x00, 0x00, 0x00, 0x12, 0x00, 0x00, 0x00
};

static unsigned int rivatv_scale_table_1 [] = {
	0x0001, 0x0002, 0x0003, 0x0004, 0x0006, 0x0008, 0x000C, 0x0010, 0x0018, 0xFFFF };
static unsigned int rivatv_scale_table_2 [] = {
	0x0000, 0x0020, 0x0040, 0x0040, 0x0060, 0x0060, 0x0080, 0x0080, 0x00A0, 0xFFFF };
static unsigned int rivatv_scale_table_3 [] = {
	0x0000, 0x0100, 0x0100, 0x0200, 0x0200, 0x0300, 0x0300, 0x0400, 0x0400, 0xFFFF };


/* ------------------------------------------------------------------------- *
 *
 * MMX optimizations
 *
 * ------------------------------------------------------------------------- */

int isMMX = 0;

#if defined (__i386__) || defined (__x86_64__)
#define cpu_detect_mmx() do {						 \
	if (test_bit (X86_FEATURE_MMX,	boot_cpu_data.x86_capability)) { \
		PRINTK_INFO ("MMX processor extension detected\n");      \
		isMMX = 1;						 \
	} } while (0)
#else
#define cpu_detect_mmx() isMMX = 0
#endif

/* ------------------------------------------------------------------------- *
 *
 * V4L part
 *
 * ------------------------------------------------------------------------- */

/* Reprogram the Riva TV scale weight based upon a MAX_WIDTH and WIDTH. */
static int rivatv_program_weights (struct rivatv_info *info, int max_width, int width)
{
	int ratio, idx, n;
	u32 reg1, reg2, val1, val2, val3;

	if (width > max_width)
		return 0;

	if (max_width == 0)
		return 1;
  
	if (width == 0) {
		ratio = 0x10000;
	} else	{
		if (max_width == 0)
			ratio = 0x10000;
		else
			ratio = (max_width << 16) / width;
	}

	/* horizontal ratio > 2.5 */
	if (ratio > 0x28000)
		idx = 4;
	/* horizontal ratio > 1.53 */
	else if (ratio > 0x187AE)
		idx = 3;
	/* horizontal ratio > 1.17 */
	else if (ratio > 0x12B85)
		idx = 2;
	/* horizontal ratio > 1 */
	else if (ratio > 0x10000)
		idx = 1;
	/* horizontal ratio <= 1 */
	else
		idx = 0;

	for (n = 0; n < 8; n++) {
		val1 = rivatv_weight_table[160 * idx + n * 20 + 0];
		val2 = rivatv_weight_table[160 * idx + n * 20 + 4];
		val3 = rivatv_weight_table[160 * idx + n * 20 + 8];
		reg1 = (((val3 & 0xFF) << 0x10) & 0x00FF0000) |
			(((val2 & 0xFF) << 0x08) & 0x0000FF00) |
			((val1 & 0xFF) & 0x000000FF);
      
		val1 = rivatv_weight_table[160 * idx + n * 20 + 12];
		val2 = rivatv_weight_table[160 * idx + n * 20 + 16];
		reg2 = (((val2 & 0xFF) << 0x08) & 0x000FF00) |
			((val1 & 0xFF) & 0x000000FF);

		/* NV_PME_HORIZ_WGHTS_A */
		VID_WR32 (info->chip.PME, 0x2004B0 + n * 4, reg1);
		/* NV_PME_HORIZ_WGHTS_B */
		VID_WR32 (info->chip.PME, 0x2004D0 + n * 4, reg2);
	}
	return 1;
}

/* Find an appropriate scale factor within one of the scale tables. */
static int rivatv_nearest_scale_factor (int scale_width, int *idx)
{
	int n = 0;

	*idx = 0xFFFF;
	while (rivatv_scale_table_1[n] != 0xFFFF) {
		if (scale_width >= rivatv_scale_table_1[n])
			*idx = n;
		n++;
	}
	if (*idx == 0xFFFF) {
		*idx = 1;
		return 0;
	}
	return 1;
}

/* Calculate scale factor for one of the Riva TV registers. */
static int rivatv_calculate_scaler_value (int max_width, int max_height, 
					  int width, int height, u32 *inc)
{
	int idx, scale_width, scale_height;

	if (width == 0)
		width = max_width;
	scale_width = max_width / width;
	if (scale_width > 24)
		scale_width = 24;
	if (rivatv_nearest_scale_factor (scale_width, &idx) == 0)
		return 0;
  
	scale_width = rivatv_scale_table_1[idx] | 
		rivatv_scale_table_2[idx] | 
		rivatv_scale_table_3[idx];
	if (height == 0)
		height = max_height;
	scale_height = ((height - 1) << 10) / (max_height - 1);
	/* vertical ratio > 1 */
	if (scale_height > 0x400)
		scale_height = 0x400;
	*inc = (scale_height << 16) | scale_width;
	return 1;
}

/* Start video transfer for NV04 architecture. */
static void rivatv_start_video_nv04 (struct rivatv_info *info)
{
	struct rivatv_port *port = &info->port;
	u32 fb_offset_a, fb_offset_b;
	u32 size, pitch, scaler;

	fb_offset_a = fb_offset_b = info->picture_offset;
	size = port->size;
	pitch = port->org_width * 2;
	
	/* Adjust values for interlaced video picture. */
	if (port->flags & VIDEO_INTERLACED) {
		fb_offset_b += pitch;
		pitch *= 2;
	}

	/* Calculate the scaler register value. */
	rivatv_calculate_scaler_value (port->max_width, port->max_height,
				       port->org_width, port->org_height, &scaler);

	/* Clear some registers. */
	VID_WR32 (info->chip.PME, 0x200418, 0);
	VID_WR32 (info->chip.PME, 0x20041C, 0);
	VID_WR32 (info->chip.PME, 0x200420, 0);

	/* Enable interrupts. */
	VID_OR32 (info->chip.PMC, 0x000140, 0x01);
	VID_OR32 (info->chip.PME, 0x200140, 0x01);

	/* Enable tasks. */
	VID_WR32 (info->chip.PME, 0x200200, 
		  (VID_RD32 (info->chip.PME, 0x200200) & 0xFFFFFFEC) | 0x12);

	/* Setup BUF0 of task A.*/
	VID_WR32 (info->chip.PME, 0x200400, fb_offset_a);
	VID_WR32 (info->chip.PME, 0x200408, pitch);
	VID_WR32 (info->chip.PME, 0x200410, size);
	VID_WR32 (info->chip.PME, 0x200424, scaler);

	/* Setup BUF1 of task A. */
	VID_WR32 (info->chip.PME, 0x200404, fb_offset_b);
	VID_WR32 (info->chip.PME, 0x20040C, pitch);
	VID_WR32 (info->chip.PME, 0x200414, size);
	VID_WR32 (info->chip.PME, 0x200428, scaler);

	/* Y-Crop lines (at top) for Task A. */
	VID_WR32 (info->chip.PME, 0x20042C, 0);

	/* Start BUF0 of Task A. */
	VID_WR32 (info->chip.PME, 0x20041C, 
		  (VID_RD32 (info->chip.PME, 0x20041C) & 0xFFFFFEFF) ^ 0x10000);
}

/* Stop video transfer for NV04 architecture. */
static void rivatv_stop_video_nv04 (struct rivatv_info *info)
{
	VID_AND32 (info->chip.PME, 0x200140, 0xFFFFFFFE);
	VID_AND32 (info->chip.PME, 0x200200, 0xFFFFFFEF);
	VID_WR32 (info->chip.PME, 0x200418, 0);
	VID_WR32 (info->chip.PME, 0x20041C, 0);
	VID_WR32 (info->chip.PME, 0x200420, 0);
}

/* Start video transfer for NV10 architecture. */
static void rivatv_start_video_nv10 (struct rivatv_info *info)
{
	struct rivatv_port *port = &info->port;
	u32 fb_offset_a, fb_offset_b;
	u32 size, pitch, height;

	fb_offset_a = fb_offset_b = info->picture_offset;
	size = port->size;
	pitch = port->org_width << 1;
	height = port->org_height;

	/* Adjust values for interlaced video picture. */
	if (port->flags & VIDEO_INTERLACED) {
		fb_offset_b += pitch;
		pitch <<= 1;
		height >>= 1;
	}

	/* NV_PME_TASKA_ME_STATE */
	VID_WR32 (info->chip.PME, 0x200458, 0);
	/* NV_PME_TASKA_SU_STATE */
	VID_WR32 (info->chip.PME, 0x20045C, 0);
	/* NV_PME_TASKA_RM_STATE */
	VID_WR32 (info->chip.PME, 0x200460, 0);
	/* NV_PME_656_CONFIG_TASKA_ONLY_ENABLED
	   NV_PME_656_CONFIG_TASKA_ENABLE */
	VID_OR32 (info->chip.PME, 0x200400, 0x41);

	VID_OR32 (info->chip.PMC, 0x000140, 0x01);
	/* NV_PME_INTR_EN_0_TASKA_NOTIFY_ENABLED */
	VID_OR32 (info->chip.PME, 0x200140, 0x01);

	rivatv_program_weights (info, port->max_width, port->org_width);

	/* NV_PME_TASKA_BUFF0_START_ADDRESS */
	VID_WR32 (info->chip.PME, 0x200440, fb_offset_a);
	/* NV_PME_TASKA_BUFF0_PITCH_VALUE */
	VID_WR32 (info->chip.PME, 0x200448, pitch);
	/* NV_PME_TASKA_BUFF0_LENGTH_VALUE */
	VID_WR32 (info->chip.PME, 0x200450, size);

	/* NV_PME_TASKA_BUFF1_START_ADDRESS */
	VID_WR32 (info->chip.PME, 0x200444, fb_offset_b);
	/* NV_PME_TASKA_BUFF1_PITCH_VALUE */
	VID_WR32 (info->chip.PME, 0x20044C, pitch);
	/* NV_PME_TASKA_BUFF1_LENGTH_VALUE */
	VID_WR32 (info->chip.PME, 0x200454, size);

	/* NV_PME_TASKA_Y_SCALE_INCR */
	VID_WR32 (info->chip.PME, 0x200468, 
		  ((height << 10) - 1) / (port->max_height - 1));
	/* NV_PME_TASKA_X_SCALE_FILTER_ENABLE
	   NV_PME_TASKA_X_SCALE_INCR */
	VID_WR32 (info->chip.PME, 0x20046C, 
		  ((port->max_width << 16) / port->org_width) | 0x80000000);

	/* NV_PME_TASKA_Y_CROP_STARTLINE */
	VID_WR32 (info->chip.PME, 0x200464, 0);

	/* NV_PME_TASKA_LINE_LENGTH_VALUE */
	if (port->org_width == 0)
		VID_WR32 (info->chip.PME, 0x2004F0, 0x04);
	else
		VID_WR32 (info->chip.PME, 0x2004F0, port->max_width << 1);
	/* NV_PME_TASKA_SU_STATE_BUFF0_IN_USE
	   NV_PME_TASKA_SU_STATE_BUFF0_FIELD */
	VID_WR32 (info->chip.PME, 0x20045C,
		  (VID_RD32 (info->chip.PME, 0x20045C) & 0xFFFFFEFF) ^ 0x10000);
}

/* Stop video transfer for NV10 architecture. */
static void rivatv_stop_video_nv10 (struct rivatv_info *info)
{
	/* NV_PME_INTR_EN_0_TASKA_NOTIFY_DISABLED 
	   NV_PME_INTR_EN_0_TASKB_NOTIFY_DISABLED */
	VID_AND32 (info->chip.PME, 0x200140, 0xFFFFFFEE);
	/* NV_PME_656_CONFIG_TASKA_ONLY_DISABLED
	   NV_PME_656_CONFIG_TASKA_DISABLED */
	VID_AND32 (info->chip.PME, 0x200400, 0xFFFFFFBE);
	/* NV_PME_TASKA_ME_STATE */
	VID_WR32 (info->chip.PME, 0x200458, 0);
	/* NV_PME_TASKA_SU_STATE */
	VID_WR32 (info->chip.PME, 0x20045C, 0);
	/* NV_PME_TASKA_RM_STATE */
	VID_WR32 (info->chip.PME, 0x200460, 0);
}

/* Compute chromakey depending on the colour depth. */
static u32 rivatv_overlay_colorkey (struct rivatv_overlay *overlay)
{
	u32 r, g, b, key = 0;

	r = (overlay->window.chromakey & 0x00FF0000) >> 16;
	g = (overlay->window.chromakey & 0x0000FF00) >> 8;
	b = overlay->window.chromakey & 0x000000FF;

	switch (overlay->buffer.depth) {
	case 15:
		key = ((r >> 3) << 10) | ((g >> 3) << 5) | ((b >> 3));
		break;
	case 16:
		key = ((r >> 3) << 11) | ((g >> 2) << 5) | ((b >> 3));
		break;
	case 24:
		key = overlay->window.chromakey & 0x00FFFFFF;
		break;
	case 32:
		key = overlay->window.chromakey;
		break;
	default:
		/* THINKME: Possible to pass a colour index for 8 bpp ? */
		DPRINTK ("invalid color depth: %d bpp\n", overlay->buffer.depth);
		break;
	}
	DPRINTK ("overlay colour key is: %08X\n", key);
	return key;
}

/* Start overlay video. */
void rivatv_overlay_start (struct rivatv_info *info) {

	struct rivatv_port *port = &info->port;
	struct rivatv_overlay *overlay = &info->overlay;
	struct video_window *window = &overlay->window;
	u32 base, pitch, size, idx, offset, xscale, yscale;
	int x, y, width, height;

	if (overlay->enable)
		return;

	DPRINTK ("starting overlay %d: %dx%d at (%d,%d)\n", 
		 overlay->index, window->width, window->height, window->x, window->y);

	base = info->picture_offset;
	size = port->size;
	pitch = port->org_width << 1;
	idx = overlay->index << 2;

	spin_lock_irq (&info->video_lock);
	switch (info->chip.arch) {
	case NV_ARCH_20:
	case NV_ARCH_10:

		/* adjust negative output window variables */
		offset = 0;
		x = window->x;
		y = window->y;
		width = window->width;
		height = window->height;
		if ((int32_t) window->x < 0) {
			x = 0;
			width = window->width + window->x;
			offset += (-window->x) << 2; 
		}
		if ((int32_t) window->y < 0) {
			y = 0;
			height = window->height + window->y;
			offset += (-window->y * port->org_width) << 2; 
		}

		/* NV_PVIDEO_BASE */
		VID_WR32 (info->chip.PVIDEO, 0x8900 + idx, base);
		/* NV_PVIDEO_LIMIT */
		VID_WR32 (info->chip.PVIDEO, 0x8908 + idx, base + size);

		/* extra code for NV20 architectures */
		if (info->chip.arch == NV_ARCH_20) {
			/* Why checking this value ? */
			if (1 || (VID_RD32 (info->chip.PVIDEO, 0x8958 + idx) & 0x01)) {
				VID_WR32 (info->chip.PVIDEO, 0x8800 + idx, base);
				VID_WR32 (info->chip.PVIDEO, 0x8808 + idx, base + size);
			}
		}

		/* NV_PVIDEO_LUMINANCE */
		VID_WR32 (info->chip.PVIDEO, 0x8910 + idx, 0x00001000);
		/* NV_PVIDEO_CHROMINANCE */
		VID_WR32 (info->chip.PVIDEO, 0x8918 + idx, 0x00001000);

		/* NV_PVIDEO_OFFSET */
		VID_WR32 (info->chip.PVIDEO, 0x8920 + idx, offset);
		/* NV_PVIDEO_SIZE_IN */
		VID_WR32 (info->chip.PVIDEO, 0x8928 + idx, (port->org_height << 16) | port->org_width);
		/* NV_PVIDEO_POINT_IN */
		VID_WR32 (info->chip.PVIDEO, 0x8930 + idx, 0x00000000);
		/* NV_PVIDEO_DS_DX_RATIO */
		VID_WR32 (info->chip.PVIDEO, 0x8938 + idx, (port->org_width << 20) / window->width);
		/* NV_PVIDEO_DT_DY_RATIO */
		VID_WR32 (info->chip.PVIDEO, 0x8940 + idx, (port->org_height << 20) / window->height);

		/* NV_PVIDEO_POINT_OUT */
		VID_WR32 (info->chip.PVIDEO, 0x8948 + idx, (y << 16) | x);
		/* NV_PVIDEO_SIZE_OUT */
		VID_WR32 (info->chip.PVIDEO, 0x8950 + idx, (height << 16) | width);

		/* NV_PVIDEO_FORMAT */
		VID_WR32 (info->chip.PVIDEO, 0x8958 + idx, pitch | 0x00100000);
		/* NV_PVIDEO_COLOR_KEY */
		overlay->colorkey = VID_RD32 (info->chip.PVIDEO, 0x8B00);
		DPRINTK ("overlay colour key has been: %08X\n", overlay->colorkey);
		VID_WR32 (info->chip.PVIDEO, 0x8B00, rivatv_overlay_colorkey (overlay));

		/* NV_PVIDEO_INTR_EN_BUFFER */
		VID_OR32 (info->chip.PVIDEO, 0x8140, 1 << idx);
		/* NV_PVIDEO_STOP */
		VID_AND32 (info->chip.PVIDEO, 0x8704, 0xFFFFFFEE);
		/* NV_PVIDEO_BUFFER */
		VID_OR32 (info->chip.PVIDEO, 0x8700, 1 << idx);
		break;

	case NV_ARCH_04:
		
		/* adjust negative output window variables */
		offset = (port->org_width - port->vld_width) & ~1;
		x = window->x;
		y = window->y;
		width = window->width;
		height = window->height;
		if ((int32_t) window->x < 0) {
			x = 0;
			width = window->width + window->x;
			offset += (-window->x * port->vld_width / window->width) << 1; 
		}
		if ((int32_t) window->y < 0) {
			y = 0;
			height = window->height + window->y;
			offset += (-window->y * port->vld_height / window->height * port->org_width) << 1;
		}

		/* NV_PVIDEO_OE_STATE */
		VID_WR32 (info->chip.PVIDEO, 0x224, 0);
		/* NV_PVIDEO_SU_STATE */
		VID_WR32 (info->chip.PVIDEO, 0x228, 0);
		/* NV_PVIDEO_RM_STATE */
		VID_WR32 (info->chip.PVIDEO, 0x22C, 0);

		/* NV_PVIDEO_OVERLAY (KEY_ON, VIDEO_OFF, FORMAT_CCIR) */
		VID_WR32 (info->chip.PVIDEO, 0x244, 0x010);

		/* NV_PVIDEO_BUFF0_START_ADDRESS */
		VID_WR32 (info->chip.PVIDEO, 0x20C + 0, base + offset);
		VID_WR32 (info->chip.PVIDEO, 0x20C + 4, base + offset);
		/* NV_PVIDEO_BUFF0_PITCH_LENGTH */
		VID_WR32 (info->chip.PVIDEO, 0x214 + 0, pitch);
		VID_WR32 (info->chip.PVIDEO, 0x214 + 4, pitch);

		/* NV_PVIDEO_WINDOW_START */
		VID_WR32 (info->chip.PVIDEO, 0x230, (y << 16) | x);
		/* NV_PVIDEO_WINDOW_SIZE */
		VID_WR32 (info->chip.PVIDEO, 0x234, (height << 16) | width);
		/* NV_PVIDEO_STEP_SIZE */
		yscale = (port->vld_height << 11) / window->height;
		xscale = (port->vld_width << 11) / window->width;
		VID_WR32 (info->chip.PVIDEO, 0x200, (yscale << 16) | xscale);

		/* NV_PVIDEO_RED_CSC_OFFSET */
		VID_WR32 (info->chip.PVIDEO, 0x280, 0x69);
		/* NV_PVIDEO_GREEN_CSC_OFFSET */
		VID_WR32 (info->chip.PVIDEO, 0x284, 0x3e);
		/* NV_PVIDEO_BLUE_CSC_OFFSET */
		VID_WR32 (info->chip.PVIDEO, 0x288, 0x89);
		/* NV_PVIDEO_CSC_ADJUST */
		VID_WR32 (info->chip.PVIDEO, 0x28C, 0x00000); /* No colour correction! */

		/* NV_PVIDEO_CONTROL_Y (BLUR_ON, LINE_HALF) */
		VID_WR32 (info->chip.PVIDEO, 0x204, 0x001);
		/* NV_PVIDEO_CONTROL_X (WEIGHT_HEAVY, SHARPENING_ON, SMOOTHING_ON) */
		VID_WR32 (info->chip.PVIDEO, 0x208, 0x111);

		/* NV_PVIDEO_FIFO_BURST_LENGTH */
		VID_WR32 (info->chip.PVIDEO, 0x23C, 0x03);
		/* NV_PVIDEO_FIFO_THRES_SIZE */
		VID_WR32 (info->chip.PVIDEO, 0x238, 0x38);

		/* NV_PVIDEO_BUFF0_OFFSET */
		VID_WR32 (info->chip.PVIDEO, 0x21C + 0, 0);
		VID_WR32 (info->chip.PVIDEO, 0x21C + 4, 0);

		/* NV_PVIDEO_KEY */
		overlay->colorkey = VID_RD32 (info->chip.PVIDEO, 0x240);
		DPRINTK ("overlay colour key has been: %08X\n", overlay->colorkey);
		VID_WR32 (info->chip.PVIDEO, 0x240, rivatv_overlay_colorkey (overlay));

		/* NV_PVIDEO_INTR_EN_0_NOTIFY_ENABLED */
		VID_OR32 (info->chip.PVIDEO, 0x140, 0x01);
		/* NV_PVIDEO_OVERLAY_VIDEO_ON */
		VID_OR32 (info->chip.PVIDEO, 0x244, 0x01);
		/* NV_PVIDEO_SU_STATE */
		VID_XOR32 (info->chip.PVIDEO, 0x228, 1 << 16);
		break;

	case NV_ARCH_03:
		break;
	}
	overlay->enable = 1;
	spin_unlock_irq (&info->video_lock);
}

/* Stop overlay video. */
void rivatv_overlay_stop (struct rivatv_info *info) {

	u32 idx;

	if (!info->overlay.enable)
		return;

	DPRINTK ("stopping overlay %d\n", info->overlay.index);
	idx = info->overlay.index << 2;

	spin_lock_irq (&info->video_lock);
	switch (info->chip.arch ) {
	case NV_ARCH_20:
	case NV_ARCH_10:
		/* NV_PVIDEO_COLOR_KEY */
		/* Xv-Extension-Hack: Restore previously saved value. */
		VID_WR32 (info->chip.PVIDEO, 0x8B00, info->overlay.colorkey);
		/* NV_PVIDEO_STOP */
		VID_OR32 (info->chip.PVIDEO, 0x8704, 0x11);
		/* NV_PVIDEO_BUFFER */
		VID_AND32 (info->chip.PVIDEO, 0x8700, ~(1 << idx));
		/* NV_PVIDEO_INTR_EN_BUFFER */
		VID_AND32 (info->chip.PVIDEO, 0x8140, ~(1 << idx));
		break;
	case NV_ARCH_04:
		/* NV_PVIDEO_KEY */
		VID_WR32 (info->chip.PVIDEO, 0x240, info->overlay.colorkey);
		/* NV_PVIDEO_OVERLAY_VIDEO_OFF */
		VID_AND32 (info->chip.PVIDEO, 0x244, ~0x01);
		/* NV_PVIDEO_INTR_EN_0_NOTIFY */
		VID_AND32 (info->chip.PVIDEO, 0x140, ~0x01);
		/* NV_PVIDEO_OE_STATE */
		VID_WR32 (info->chip.PVIDEO, 0x224, 0);
		/* NV_PVIDEO_SU_STATE */
		VID_WR32 (info->chip.PVIDEO, 0x228, 0);
		/* NV_PVIDEO_RM_STATE */
		VID_WR32 (info->chip.PVIDEO, 0x22C, 0);
		break;
	case NV_ARCH_03:
		break;
	}
	info->overlay.enable = 0;
	spin_unlock_irq (&info->video_lock);
}

/* Schedules the next video picture to be transfered from the video decoder
   into the framebuffer.  Thus we can ensure that no additional video picture
   runs into the currently captured (grabdisplay) or displayed (overlay).
   This hopefully fixes the interlace actefact problem for larger resolutions. */
static void rivatv_schedule_next (struct rivatv_info *info)
{
	if (info->port.state_buf_notify == 0)
		return;

	switch (info->chip.arch) {
	case NV_ARCH_03:
	case NV_ARCH_04:
		VID_XOR32 (info->chip.PME, 0x200420, info->port.state_buf_notify);
		VID_XOR32 (info->chip.PME, 0x20041C, info->port.state_buf_usage);
		break;
	case NV_ARCH_10:
	case NV_ARCH_20:
		/* toggle NV_PME_TASKA_RM_STATE_BUFF1_INTR_NOTIFY */
		VID_XOR32 (info->chip.PME, 0x200460, info->port.state_buf_notify);
		/* toggle NV_PME_TASKA_SU_STATE_BUFF0_IN_USE */
		VID_XOR32 (info->chip.PME, 0x20045C, info->port.state_buf_usage);
		break;
	}
	info->port.state_buf_notify = 0;
	info->port.state_buf_usage = 0;
}


/* The PCI interrupt handler. */
static void rivatv_interrupt (int irq, void *dev_id, struct pt_regs * regs) {

	int both_fields_done = 0;
	struct rivatv_info *info;
	struct video_device *dev = dev_id;
	u32 state_OE, state_SU, state_RM, buffer_use_state, state_ME, buffer_intr_state, idx;
    
	if (dev == NULL) {
		PRINTK_ERR ("IRQ %d for invalid device\n", irq);
		return;
	}
	info = (struct rivatv_info *) (dev->priv);

	spin_lock (&info->video_lock);
	info->interrupts++;

	switch (info->chip.arch) {
	case NV_ARCH_20:
	case NV_ARCH_10:
		
		/* Reset the overlay interrupt register. 
		   This interrupt happens to occur if overlay output is stopped. */

		idx = info->overlay.index << 2;
		/* NV_PVIDEO_INTR_BUFFER_0_PENDING */
		if (VID_RD32 (info->chip.PVIDEO, 0x8100) & (1 << idx)) {
			info->overlay.interrupts++;
			/* NV_PVIDEO_INTR_BUFFER_X_RESET */
			VID_OR32 (info->chip.PVIDEO, 0x8100, (1 << idx));
		}

		/* NV_PME_INTR_EN_0_TASKA_NOTIFY_ENABLED */
		if (!(VID_RD32 (info->chip.PME, 0x200140) & 0x01)) {
			spin_unlock (&info->video_lock);
			return;
		}

		/* NV_PME_INTR_0_TASKA_NOTIFY_RESET */
		VID_OR32 (info->chip.PME, 0x200100, 0x01);

		/* Evaluate task A status registers. */

		/* NV_PME_TASKA_ME_STATE */
		state_ME = VID_RD32 (info->chip.PME, 0x200458);
		/* NV_PME_TASKA_SU_STATE */
		state_SU = VID_RD32 (info->chip.PME, 0x20045C);
		/* NV_PME_TASKA_RM_STATE */
		state_RM = VID_RD32 (info->chip.PME, 0x200460);

		/* NV_PME_TASKA_ME_STATE_BUFF01_IN_USE ^ NV_PME_TASKA_SU_STATE_BUFF01_IN_USE */
		buffer_use_state = state_ME ^ state_SU;
		/* NV_PME_TASKA_ME_STATE_BUFF01_INTR_NOTIFY ^ NV_PME_TASKA_RM_STATE_BUFF01_INTR_NOTIFY */
		buffer_intr_state = state_ME ^ state_RM;

		/* BUF0 in use or not in use and BUF0 notified */
		if (!(buffer_use_state & 0x10000) && (buffer_intr_state & 0x01)) {
			info->interrupt_count++;
			if (info->port.flags & VIDEO_INTERLACED) {
				/* toggle NV_PME_TASKA_RM_STATE_BUFF0_INTR_NOTIFY */
				VID_XOR32 (info->chip.PME, 0x200460, 0x01);
				/* toggle NV_PME_TASKA_SU_STATE_BUFF1_IN_USE */
				VID_XOR32 (info->chip.PME, 0x20045C, 0x100000);
			} else {
				info->port.state_buf_notify = 0x01;
				info->port.state_buf_usage  = 0x100000;
				both_fields_done = 1;
			}
		}
		/* BUF1 in use or not in use and BUF1 notified */
		else if (!(buffer_use_state & 0x100000) && (buffer_intr_state & 0x10)) {
			info->interrupt_count++;
			info->port.state_buf_notify = 0x10;
			info->port.state_buf_usage  = 0x010000;
			both_fields_done = 1;
		}
		if (info->overlay.enable)
			rivatv_schedule_next (info);
			
		break;

	case NV_ARCH_04:
		idx = info->overlay.index << 2;

		/* NV_PVIDEO_INTR_0_NOTIFY_PENDING */
		if (VID_RD32 (info->chip.PVIDEO, 0x100) & 0x01) {
			/* NV_PVIDEO_INTR_0_NOTIFY_RESET */
			VID_OR32 (info->chip.PVIDEO, 0x100, 0x01);
			info->overlay.interrupts++;

			/* Evaluate status registers. */
			state_OE = VID_RD32 (info->chip.PVIDEO, 0x224);
			state_SU = VID_RD32 (info->chip.PVIDEO, 0x228);
			state_RM = VID_RD32 (info->chip.PVIDEO, 0x22C);
			buffer_use_state = state_OE ^ state_SU;  /* true if different */
			buffer_intr_state = state_OE ^ state_RM; /* true if different */
			if ((buffer_use_state & 0x10000) && (buffer_intr_state & 0x10)) {
				VID_XOR32 (info->chip.PVIDEO, 0x228, 0x110000);
				VID_XOR32 (info->chip.PVIDEO, 0x22C, 0x11);
				rivatv_schedule_next (info);
			}
		}

	case NV_ARCH_03:
		if (!(VID_RD32 (info->chip.PME, 0x200140) & 0x01)) {
			spin_unlock (&info->video_lock);
			return;
		}

		/* Reset the interrupt register. */
		VID_OR32 (info->chip.PME, 0x200100, 0x01);

		/* Evaluate task A status registers. */
		state_ME = VID_RD32 (info->chip.PME, 0x200418);
		state_SU = VID_RD32 (info->chip.PME, 0x20041C);
		state_RM = VID_RD32 (info->chip.PME, 0x200420);

		buffer_use_state = state_ME ^ state_SU;
		buffer_intr_state = state_ME ^ state_RM;
      
		if (!(buffer_use_state & 0x10000) && (buffer_intr_state & 0x01)) {
			info->interrupt_count++;
			if (info->port.flags & VIDEO_INTERLACED) {
				VID_XOR32 (info->chip.PME, 0x200420, 0x01);
				VID_XOR32 (info->chip.PME, 0x20041C, 0x100000);
			} else {
				info->port.state_buf_notify = 0x01;
				info->port.state_buf_usage  = 0x100000;
				both_fields_done = 1;
			}
		} else if (!(buffer_use_state & 0x100000) && (buffer_intr_state & 0x10)) {
			info->interrupt_count++;
			info->port.state_buf_notify = 0x10;
			info->port.state_buf_usage  = 0x010000;
			both_fields_done = 1;
		}
		break;
	}

	/* Return here for overlay'ing. */
	if (info->overlay.enable) {
		spin_unlock (&info->video_lock);
		return;
	}

	/* Schedule the bottom half here. */
	if (both_fields_done) {
		if (info->capture_schedule < info->capture_buffers) {
			if (queue_task (&info->capture_queue.task, &tq_immediate)) {
				info->capture_schedule++;
				mark_bh (IMMEDIATE_BH);
			}
		}
	}
	else if (jiffies > info->last_jiffies + HZ / 10) {
		if (info->capture_buffer != NULL && info->capture_schedule < info->capture_buffers) {
			if (queue_task (&info->capture_queue.task, &tq_immediate)) {
				info->capture_schedule++;
				mark_bh (IMMEDIATE_BH);
			}
		}
	}

	spin_unlock (&info->video_lock);
	return;
}

/* Enable PFB (Framebuffer), PVIDEO (Overlay unit) and PME (Mediaport) if neccessary. */
static void __init rivatv_enable_PMEDIA (struct rivatv_info *info)
{
	u32 reg;

	/* switch off interrupts once for a while */
	VID_WR32 (info->chip.PME, 0x200140, 0x00);
	VID_WR32 (info->chip.PMC, 0x000140, 0x00);

	reg = VID_RD32 (info->chip.PMC, 0x000200);
	if ((reg & 0x10100010) != 0x10100010) {
		PRINTK_INFO ("PMEDIA, PVIDEO and PFB disabled, enabling...\n");
		VID_OR32 (info->chip.PMC, 0x000200, 0x10100010);
	}

	/* re-enable interrupts again */
	VID_WR32 (info->chip.PMC, 0x000140, 0x01);
	VID_WR32 (info->chip.PME, 0x200140, 0x01);
}

/* Install the interrupt routine for video. */
static int __init rivatv_install_irq (struct rivatv_info *info)
{
	int err;

	err = request_irq (info->pci->irq, rivatv_interrupt,
			   SA_INTERRUPT | SA_SHIRQ, "rivatv", &info->video);
	if (err < 0) {
		PRINTK_ERR ("unable to get IRQ %d (errno = %d)\n", info->pci->irq, err);
		return err;
	}

	/* Enable IRQs on PME and PMC. */
	rivatv_enable_PMEDIA (info);

	DPRINTK ("successfully requested IRQ %d\n", info->pci->irq);
	return 0;
}

/* Uninstall the interrupt routine for video. */
static int __exit rivatv_deinstall_irq (struct rivatv_info *info)
{
	VID_WR32 (info->chip.PME, 0x200140, 0);
	free_irq (info->pci->irq, &info->video);
	DPRINTK ("successfully freed IRQ %d\n", info->pci->irq);
	return 0;
}

struct rivatv_video_standard {
	/* video standard constant */
	int standard;
	/* maximum width in pixels */
	int width;
	/* maximum height in pixels */
	int height;
};

static struct rivatv_video_standard rivatv_standard[3] = {
	{ VIDEO_MODE_PAL,   704, 576 },
	{ VIDEO_MODE_NTSC,  704, 480 },
	{ VIDEO_MODE_SECAM, 704, 576 }
};

void rivatv_get_capability (struct rivatv_info *info, struct video_capability *cap)
{
	if (info->overlay.enable) {
		cap->maxwidth = info->overlay.buffer.width;
		cap->maxheight = info->overlay.buffer.height;
	} else {
		cap->maxwidth = rivatv_standard[info->format.norm].width;
		cap->maxheight = rivatv_standard[info->format.norm].height;
		if (info->chip.arch == NV_ARCH_03) {
			cap->maxwidth /= 2;	/* Is it really true? */
			cap->maxheight /= 2;
		}
	}
	if (info->tuner_required != -1 && info->i2c->tuner != NULL) {
		cap->type |= VID_TYPE_TUNER;
	}
	if (info->audio_required != -1 && info->i2c->audio_processor != NULL) {
		cap->audios = 1;
	}
}

/* Setup size of video decoder image in framebuffer for overlay. */
void rivatv_set_overlay_format (struct rivatv_info *info)
{
	info->format.overlay = 1;
	info->format.width = rivatv_standard[info->format.norm].width;
	info->format.height = rivatv_standard[info->format.norm].height;

	switch (info->chip.arch) {
	case NV_ARCH_03:
		info->format.width /= 2;	/* Is it really true? */
		info->format.height /= 2;
		break;
	case NV_ARCH_04:
		if (info->overlay.window.width < info->format.width)
			info->format.width = info->overlay.window.width;
		if (info->overlay.window.height < info->format.height)
			info->format.height = info->overlay.window.height;
		break;
	}
}

/* Scales a UYVY video buffer DOWN. */
static void rivatv_soft_scale (char *src_buf, char *dst_buf,
			       int src_width, int src_height, int src_offset, int src_pitch,
			       int dst_width, int dst_height)
{
#if defined (__i386__) || defined (__x86_64__)
	static int count;

	__asm__ __volatile__(
	"   movl      %0, %%esi		\n"
	"   shll      $1, %6		\n"
	"   shll      $1, %7		\n"
	"   addl      %6, %%esi		\n"
	"   movl      %1, %%edi		\n"
	"   movl      %2, %%edx		\n"
	"   movl      %5, %%eax		\n"
	"   movl      %%eax, %8		\n"
	"   movl      %3, %%ebx		\n"
	"   shrl      $1, %%ebx		\n"
	"1: addl      %5, %%ebx		\n"
	"   cmpl      %3, %%ebx		\n"
	"   jae	      2f		\n"
	"   addl      %7, %%esi		\n"
	"   jmp	      1b		\n"
	"2: subl      %3, %%ebx		\n"
	"   movl      %4, %%ecx		\n"
	"   movl      %%edx, %%eax	\n"
	"   shrl      $1, %%ecx		\n"
	"   shrl      $1, %%eax		\n"
	"   pushl     %%esi		\n"
	"3: addl      %4, %%eax		\n"
	"   cmpl      %%edx, %%eax	\n"
	"   jae	      4f		\n"
	"   addl      $4, %%esi		\n"
	"   jmp	      3b		\n"
	"4: movsl			\n"
	"   subl      %%edx, %%eax	\n"
	"   decl      %%ecx		\n"
	"   jnz	      3b		\n"
	"   popl      %%esi		\n"
	"   addl      %7, %%esi		\n"
	"   decl      %8		\n"
	"   jnz	      1b		\n"
	/* output */		:
	/* input */		: "g" (src_buf),	"g" (dst_buf),
				  "g" (src_width),	"g" (src_height),
				  "g" (dst_width),	"g" (dst_height),
				  "g" (src_offset),	"g" (src_pitch),
				  "g" (count)
	/* clobber registers */ : "cc", "eax", "ebx", "ecx", "edx", "esi", "edi");
#else
	int width, height, w, h;
	u32 *src = (u32 *) src_buf, *dst = (u32 *) dst_buf, *buf;
	
	src += src_offset >> 1;
	h = dst_height;
	height = src_height >> 1;
	while (h) {
		height += dst_height;
		if (height >= src_height) {
			h--;
			height -= src_height;
			w = dst_width >> 1;
			width = src_width >> 1;
			buf = src;
			while (w) {
				width += dst_width;
				if (width >= src_width) {
					w--;
					*dst++ = *buf;
					width -= src_width;
				}
				buf++;
			}
		}
		src += src_pitch >> 1;
	}
#endif /* __i386__ */
}

/* Transfer a single frame. */
static void rivatv_transfer_frame (struct rivatv_info *info, char *src, char *dst,
				   int src_width, int src_height, 
				   int dst_width, int dst_height,
				   int src_fmt, int dst_fmt)
{
	line_convert_t line_convert = rivatv_convert[src_fmt].converter[dst_fmt];
	plane_convert_t plane_convert = (plane_convert_t) line_convert;

	if (line_convert == NULL) {
		PRINTK_ERR ("no such converter: %s -> %s\n", 
			    rivatv_convert[src_fmt].name, rivatv_convert[dst_fmt].name);
		return;
	}

	if (info->chip.arch == NV_ARCH_03 || info->chip.arch == NV_ARCH_04) {
		rivatv_soft_scale (src, info->data, info->port.vld_width, src_height, 
				   (info->port.org_width - info->port.vld_width) >> 1, 
				   info->port.org_width, dst_width, dst_height);
		if (rivatv_convert[dst_fmt].planar)
			plane_convert (info->data, dst, src_width, src_height);
		else
			line_convert (info->data, dst, src_width * src_height);
	} else {
		if (rivatv_convert[dst_fmt].planar)
			plane_convert (src, dst, src_width, src_height);
		else
			line_convert (src, dst, src_width * src_height);
	}
}

/* Checks whether the active video decoder can output requested palette format. */
int rivatv_check_source (struct rivatv_info *info, struct video_mmap *vm) {

	struct rivatv_source * source = info->source;
	struct rivatv_format * format = &info->format;
	int result;

	if (format->format == vm->format)
		return 0;

	DPRINTK ("capture palette changed: %s -> %s\n",
		 rivatv_convert[format->format].name,
		 rivatv_convert[vm->format].name);
	format->format = vm->format;

	if (source->format == format->format)
		return 0;

#if 0
        /* nVidia cards handle UYVY only, possibly they change it in future */
	for (n = 0; source->formats[n] != -1; n++) {
		if (source->formats[n] == format->format) {
			result = source->format = format->format;
			DPRINTK ("decoder output set to %s\n", rivatv_convert[result].name);
			return result;
		}
	}
#endif

	/* set default UYVY format if necessary */
	result = source->format = source->formats[0];
	DPRINTK ("decoder output set to default %s\n", rivatv_convert[result].name);
	return result;
}

static void rivatv_bh (struct rivatv_info *info);

#ifndef INIT_TQUEUE
#define INIT_TQUEUE(_tq, _routine, _data)                       \
        do {                                                    \
                INIT_LIST_HEAD(&(_tq)->list);                   \
                (_tq)->sync = 0;                                \
                PREPARE_TQUEUE((_tq), (_routine), (_data));     \
        } while (0)
#endif
#ifndef PRAPARE_TQUEUE
#define PREPARE_TQUEUE(_tq, _routine, _data)                    \
        do {                                                    \
                (_tq)->routine = _routine;                      \
                (_tq)->data = _data;                            \
        } while (0)
#endif


/* Initialize the capture queue. */
static void rivatv_reset_queue (struct rivatv_info *info)
{
	int n;

	/* wait until task queue is empty */
	wait_event_interruptible (info->capture_queue.wait, info->capture_schedule <= 0);

	/* zero out everything */
	memset (&info->capture_queue, 0, sizeof (struct rivatv_cap_queue));

	/* initialize task queue */
	INIT_TQUEUE (&info->capture_queue.task, (void (*) (void *)) rivatv_bh, info);

	/* initialize capture queue */
	init_waitqueue_head (&info->capture_queue.wait);

	/* initialize capture buffers */
	memset (&info->capbuf, 0, capbuffers * sizeof (struct rivatv_cap_buffer));
	for (n = 0; n < capbuffers; n++)
		info->capbuf[n].status = RIVATV_UNUSED;
}

/* Add a buffer to capture queue. */
int rivatv_add_queue (struct rivatv_info *info, struct video_mmap *vm)
{
	struct rivatv_cap_buffer *cap;
	struct rivatv_cap_queue *queue = &info->capture_queue;
	struct rivatv_format *format = &info->format;

	/* did the user already mmap() ? */
	if (info->capture_buffer == NULL) {
		DPRINTK ("MMAP buffer %d not available\n", vm->frame);
		return -EAGAIN;
	}

	/* check frame range */
	if (vm->frame >= info->capture_buffers || vm->frame < 0)
		return -EINVAL;

	/* check width, height and palette format */
	if (vm->height < RIVATV_MIN_HEIGHT || vm->width < RIVATV_MIN_WIDTH ||
	    vm->height > RIVATV_MAX_HEIGHT || vm->width > RIVATV_MAX_WIDTH)
		return -EINVAL;
	if (!RIVATV_PALETTE_SUPPORTED (vm->format))
		return -EINVAL;

	spin_lock_irq (&info->queue_lock);
	if (queue->fill >= info->capture_buffers) {
		spin_unlock_irq (&info->queue_lock);
		return -EAGAIN;
	}

	/* initialize capture structure */
	cap = &info->capbuf[vm->frame];
	cap->format = vm->format;
	cap->width = vm->width;
	cap->height = vm->height;
	cap->status = RIVATV_GRABBING;
	cap->size = info->capture_buffer_size;
	cap->addr = info->capture_buffer + info->capture_buffer_size * vm->frame;

	/* setup format buffer */
	format->format = vm->format;
	format->width = vm->width;
	format->height = vm->height;
	format->size = (format->width * format->height * rivatv_convert[format->format].bpp) >> 3;

	/* update capture queue */
	if (queue->fill)
		queue->head = (queue->head + 1) % RIVATV_MAX_CAPTURE_BUFFERS;
	queue->index[queue->head] = vm->frame;
	queue->fill++;
	spin_unlock_irq (&info->queue_lock);
	return 0;
}

/* Remove a buffer from capture queue at tail position. */
static void rivatv_del_queue (struct rivatv_info *info)
{
	struct rivatv_cap_queue *queue = &info->capture_queue;

	if (queue->fill == 0) {
		DPRINTK ("assert (queue->fill != 0) failed\n");
		return;
	}

	/* update queue */
	queue->fill--;
	if (queue->fill)
		queue->tail = (queue->tail + 1) % RIVATV_MAX_CAPTURE_BUFFERS;
}

/* Capture a single frame. */
static void rivatv_capture_frame (struct rivatv_info *info, ulong io_base)
{
	char *buffer;
	int frame;
	struct rivatv_cap_queue *queue = &info->capture_queue;
	struct rivatv_cap_buffer *cap;
	struct rivatv_port *port = &info->port;

	spin_lock (&info->queue_lock);
	if (queue->fill == 0) {
		/* nothing to do */
		spin_unlock (&info->queue_lock);
		return;
	}

	frame = queue->index[queue->tail];
	cap = &info->capbuf[frame];

	if (io_base) {
		buffer = info->capture_buffer +	frame * info->capture_buffer_size;

		rivatv_transfer_frame (info, (void *) (io_base + info->vbi_region_size), buffer,
				       port->org_width, port->org_height,
				       cap->width, cap->height,
				       info->source->format, cap->format);
	}

	cap->status = RIVATV_READY;
	rivatv_del_queue (info);
	spin_unlock (&info->queue_lock);
}

/* Setup the capture buffer. */
static int __init rivatv_setup_capture (struct rivatv_info *info)
{
	if (!info->data) {
		if ((info->data = vmalloc_32 (RIVATV_RAW_CAPTURE_BUFSIZE)) == NULL) {
			PRINTK_ERR ("unable to allocate YUV capture buffer (%ld kb)\n",
				    RIVATV_RAW_CAPTURE_BUFSIZE / 1024);
			return -ENOMEM;
		}
		PRINTK_INFO ("allocated YUV capture buffer (%ld kb)\n", RIVATV_RAW_CAPTURE_BUFSIZE / 1024);
	}
	return 0;
}

/* Cleanup the capture buffer. */
static void __exit rivatv_cleanup_capture (struct rivatv_info *info)
{
	if (info->data) {
		PRINTK_INFO ("freeing YUV capture buffer (%ld kb)\n", RIVATV_RAW_CAPTURE_BUFSIZE / 1024);
		vfree (info->data);
		info->data = NULL;
	}
}

/* Configure the VBI region size depending on the video format. */
static void rivatv_config_vbi (struct rivatv_info *info)
{
	info->vbi_region_size = 0; // info->port.org_width << 1;
}

/* Setup default video format. */
static void __init rivatv_set_default_format (struct rivatv_format *format)
{
	format->norm = VIDEO_MODE_PAL;
	format->width = 352;
	format->height = 288;
}

/* Startup video. */
int rivatv_video_start (struct rivatv_info *info) 
{
	int width, height;
	struct rivatv_port * port = &info->port;

	if (info->capture_started)
		return 0;

	width = info->format.width;
	height = info->format.height;
	memset (port, 0, sizeof (struct rivatv_port));

	if (info->format.norm == VIDEO_MODE_NTSC) {
		port->max_width = 720;
		port->max_height = 240;
		port->size = 720 * 502 * 2;
		port->org_height = 60;
		port->vld_height = 60;
	} else if (info->format.norm == VIDEO_MODE_PAL) {
		port->max_width = 720;
		port->max_height = 288;
		port->size = 720 * 576 * 2;
		port->org_height = 72;
		port->vld_height = 72;
	}

	/* Ensure alignment of decoder width and height. Necessary for NV03 and NV04. */
	if (info->chip.arch == NV_ARCH_03 || info->chip.arch == NV_ARCH_04) {

		port->org_width = 90;
		port->vld_width = 88;
		if (info->format.overlay) {
			/* Prepare for overlay (choose <= requested size) -> UP scale only */
			while ((port->vld_width << 1) <= width) {
				port->org_width <<= 1;
				port->vld_width <<= 1;
			}
			while ((port->vld_height << 1) <= height) {
				port->org_height <<= 1;
				port->vld_height <<= 1;
			}
		} else {
			/* Prepare for capture (choose >= requested size) -> DOWN scale only */
			while (port->org_width < width) {
				port->org_width <<= 1;
				port->vld_width <<= 1;
			}
			while (port->org_height < height) {
				port->org_height <<= 1;
				port->vld_height <<= 1;
			}
		}
		port->org_width &= ~7;
		DPRINTK ("configured port: %dx%d -> %dx%d\n", width, height, 
			 port->org_width, port->org_height);
	} else {
		port->org_height = port->vld_height = height;
		port->org_width = port->vld_width = width;
	}

	port->height = height;
	port->width = width;

	/* Adjust field values. */
	if (port->org_height > port->max_height) {
		port->flags |= VIDEO_INTERLACED;
	} else {
		port->flags &= ~VIDEO_INTERLACED;
	}

	info->format.overlay = 0;
	rivatv_config_vbi (info);

	DPRINTK ("starting video capture\n");
	spin_lock_irq (&info->video_lock);
	switch (info->chip.arch) {
	case NV_ARCH_20:
	case NV_ARCH_10:
		rivatv_start_video_nv10 (info);
		break;
	case NV_ARCH_04:
	case NV_ARCH_03:
		rivatv_start_video_nv04 (info);
		break;
	}
	spin_unlock_irq (&info->video_lock);
	info->capture_started = 1;
	info->last_jiffies = jiffies;
	return 0;
}

/* Stop the video. */
int rivatv_video_stop (struct rivatv_info *info)
{
	struct rivatv_port *port = &info->port;

	if (info->capture_started == 0)
		return 0;

	memset (port, 0, sizeof (struct rivatv_port));
	DPRINTK ("stopping video capture\n");
	spin_lock_irq (&info->video_lock);
	switch (info->chip.arch) {
	case NV_ARCH_20:
	case NV_ARCH_10:
		rivatv_stop_video_nv10 (info);
		break;
	case NV_ARCH_04:
	case NV_ARCH_03:
		rivatv_stop_video_nv04 (info);
		break;
	}
	spin_unlock_irq (&info->video_lock);
	info->capture_started = 0;
	return 0;
}

/* Checks whether the the current capture format is valid or has been changed. */
int rivatv_check_format (struct rivatv_info *info, struct video_mmap *vm)
{
	struct rivatv_format *format = &info->format;

	if (format->width != vm->width || format->height != vm->height) {
		DPRINTK ("capture resolution changed: %dx%d -> %dx%d\n", 
			 format->width, format->height, vm->width, vm->height);
		format->width = vm->width;
		format->height = vm->height;
		return 1;
	}
	return 0;
}

/* The bottom half of the interrupt service routine. */
static void rivatv_bh (struct rivatv_info *info)
{
	spin_lock_irq (&info->video_lock);

	/* capture this frame */
	if (jiffies > info->last_jiffies + HZ / 10)
		rivatv_capture_frame (info, (ulong) NULL);
	else
		rivatv_capture_frame (info, (ulong) info->picture_base);

	info->capture_schedule--;
	info->last_jiffies = jiffies;
	rivatv_schedule_next (info);
	
	spin_unlock_irq (&info->video_lock);

	/* finally wake up the sync ioctl */
	wake_up_interruptible (&info->capture_queue.wait);
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 19)
/* Converts a kernel virtual address returned by vmalloc_32() into the
   a physical page. */
static struct page * vmalloc_to_page (void *address)
{
	pgd_t *pgd;  /* page directory entry */
	pmd_t *pmd;  /* page middle directory entry */
	pte_t *pte;  /* page table entries */
	ulong va = VMALLOC_VMADDR ((ulong) address);
	struct page *page = NULL;
	
	/* get the page directory for a linear kernel address */
	pgd = pgd_offset_k (va);

	/* page directory entry initialized ? */
	if (!pgd_none (*pgd)) {
		/* get the page middle directory */
		pmd = pmd_offset (pgd, va);
		/* entry already set ? */
		if (!pmd_none (*pmd)) {
			/* get a pointer to the page table entry */
			pte = pte_offset (pmd, va);
			/* any valid refering page in physical memory ? */
			if (pte_present (*pte)) {
				/* assign page return value */
				page = pte_page (*pte);
			}
		}
	}
	return page;
}
#endif

/* Converts a kernel virtual address returned by vmalloc_32() into the
   a physical address. */
static unsigned long vmalloc_to_phys_addr (unsigned long address)
{
	unsigned long ret;

        /* get the address the page is refering to */
	ret = (unsigned long) page_address (vmalloc_to_page ((void *) address));
	/* add the offset within the page to the page address */
	ret |= (address & ~PAGE_MASK);
	return virt_to_phys ((void *) ret);
}

/* Allocate an arena of size bytes. Return non-zero on errors. */
static int rivatv_alloc_arena (struct rivatv_info *info, ulong size)
{
	ulong virt_addr;
	void *ptr;
	
	/* get a memory area that is only virtually continuous */
	if ((ptr = vmalloc_32 (size)) == NULL)
		return -1;

	DPRINTK ("allocated %ld kb MMAP buffer space\n", size / 1024);

	/* reserve all pages and initialize these */
	virt_addr = (ulong) ptr;
	memset (ptr, 0, size);
	while (size > 0) {
		SetPageReserved (vmalloc_to_page ((void *) virt_addr));
		size -= PAGE_SIZE;
		virt_addr += PAGE_SIZE;
	}

	/* save this address */
	info->capture_buffer = ptr;
	return 0;
}

/* Free an arena previously reserved by rivatv_alloc_arena(). */
static void __exit rivatv_free_arena (struct rivatv_info *info)
{
	ulong virt_addr, size;

	/* unreserve all pages */
	if (info->capture_buffer) {
		size = info->capture_buffer_size * info->capture_buffers;
		virt_addr = (ulong) info->capture_buffer;
		DPRINTK ("freeing MMAP buffer space (%ld kb)\n", size / 1024);
		while (size > 0) {
			ClearPageReserved (vmalloc_to_page ((void *) virt_addr));
			size -= PAGE_SIZE;
			virt_addr += PAGE_SIZE;
		}
		vfree (info->capture_buffer);
	}

	/* and free the area */
	info->capture_buffer = NULL;
	info->capture_buffer_size = 0;
}

/* Unmaps the capture buffer in kernel from virtual user space. */
static void __exit rivatv_munmap (struct rivatv_info *info)
{
	rivatv_free_arena (info);
}

/* Maps the capture buffer in kernel to the virtual user space address ADR. 
   Allocate the capture buffer first if necessary. */
int rivatv_mmap (struct rivatv_info *info, struct vm_area_struct *vma, ulong adr, ulong size)
{
	ulong start = adr;
	ulong vsize, virt_addr;

	/* check the size */
	vsize = capbuffers * RIVATV_CAPTURE_BUFSIZE;
	if (size > vsize)
		return -EINVAL;

	if (!info->capture_buffer) {
		/* allocate capture buffer */
		if (rivatv_alloc_arena (info, vsize) != 0) {
			PRINTK_ERR ("unable to allocate capture buffer (%ld kb)\n", vsize);
			return -ENOMEM;
		}
		info->capture_buffers = capbuffers;
		info->capture_buffer_size = RIVATV_CAPTURE_BUFSIZE;
	}

	/* map the capture buffer to user space page by page */
	virt_addr = (ulong) info->capture_buffer;
	vsize = size;
	while (size > 0) {
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
		if (remap_page_range (start, vmalloc_to_phys_addr (virt_addr),
				      PAGE_SIZE, PAGE_SHARED)) {
#else
		if (remap_page_range (vma, start, vmalloc_to_phys_addr (virt_addr),
				      PAGE_SIZE, PAGE_SHARED)) {
#endif
			PRINTK_ERR ("failed to put MMAP buffer to user space\n");
			return -EAGAIN;
		}
		start += PAGE_SIZE;
		virt_addr += PAGE_SIZE;
		size -= PAGE_SIZE;
	}

	/* state this in kernel messages */
	DPRINTK ("MMAP buffer available in user space (%ld kb)\n", vsize / 1024);
	return 0;
}

/* Synchronize function for mmap() grabbing. */
int rivatv_sync (struct rivatv_info *info, int frame)
{
	int ret = 0;
	struct rivatv_cap_buffer *cap;
	struct rivatv_cap_queue *queue;

	/* check the frames range */
	if (frame < 0 || frame >= info->capture_buffers)
		return -EINVAL;

	cap = &info->capbuf[frame];
	queue = &info->capture_queue;

sync_loop:
	spin_lock_irq (&info->queue_lock);
	switch (cap->status) {
	case RIVATV_UNUSED:		/* grabbing buffer unused */
		ret = -EINVAL;
		break;
	case RIVATV_GRABBING:		/* grabbing buffer currently in use */
		spin_unlock_irq (&info->queue_lock);
		if (wait_event_interruptible (queue->wait, cap->status != RIVATV_GRABBING)) {
			return -EINTR;
		}
		goto sync_loop;
	case RIVATV_READY:		/* grabbing buffer ready */
		break;
	case RIVATV_ERROR:		/* error in grabbing buffer */
		ret = -EIO;
		break;
	default:
		DPRINTK ("this should never happen\n");
		ret = -EIO;
		break;
	}
	if (!ret)
		cap->status = RIVATV_UNUSED;
	spin_unlock_irq (&info->queue_lock);
	return ret;
}

static int __init riva_setup_video_dev (struct rivatv_info *info,
					const struct rivatv_initdata *v4l_init)
{
	int err;

	memcpy (&info->video, v4l_init->video, sizeof (info->video));
	memcpy (&info->vbi, v4l_init->vbi, sizeof (info->vbi));
	info->video.priv = info;
	info->vbi.priv = info;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 5)
	if ((err = video_register_device (&info->video, VFL_TYPE_GRABBER)) < 0)
#else
	if ((err = video_register_device (&info->video, VFL_TYPE_GRABBER, -1)) < 0)
#endif
		return err;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 5)
	if ((err = video_register_device (&info->vbi, VFL_TYPE_VBI)) < 0) {
#else
	if ((err = video_register_device (&info->vbi, VFL_TYPE_VBI, -1)) < 0) {
#endif
		video_unregister_device (&info->video);
		return err;
	}

	if ((err = rivatv_setup_capture (info)) != 0) {
		video_unregister_device (&info->vbi);
		video_unregister_device (&info->video);
		return err;
	}

	rivatv_set_default_format (&info->format);
	rivatv_reset_queue (info);
	if ((err = rivatv_install_irq (info)) != 0) {
		video_unregister_device (&info->vbi);
		video_unregister_device (&info->video);
		return err;
	}

	PRINTK_INFO ("Video4Linux device driver registered\n");
	return 0;
}

static void __exit riva_release_video_dev (struct rivatv_info *info)
{
	rivatv_deinstall_irq (info);
	rivatv_video_stop (info);
	rivatv_reset_queue (info);
	rivatv_munmap (info);
	rivatv_cleanup_capture (info);

	video_unregister_device (&info->vbi);
	video_unregister_device (&info->video);
}

int __init rivatv_register_v4l (const struct rivatv_initdata *driver_init)
{
	int err;
	struct rivatv_info *info = riva_board;
	
	if (info == NULL)
		return -ENODEV;
		
	err = riva_setup_video_dev (info, driver_init);
	if (err)
		return err;

	return 0;
}

void __exit rivatv_unregister_v4l (void)
{
	struct rivatv_info *info = riva_board;
	
	if (info == NULL)
		return;
	riva_release_video_dev (info);
}


/* ------------------------------------------------------------------------- *
 *
 * basic chip specific functions
 *
 * ------------------------------------------------------------------------- */

static void rivatv_lock_nv03 (struct rivatv_chip *chip, int LockUnlock)
{
	VID_WR08 (chip->PVIO, 0x3C4, 0x06);
	VID_WR08 (chip->PVIO, 0x3C5, LockUnlock ? 0x99 : 0x57);
}

static void rivatv_lock_nv04 (struct rivatv_chip *chip, int LockUnlock)
{
	VID_WR08 (chip->PCIO, 0x3C4, 0x06);
	VID_WR08 (chip->PCIO, 0x3C5, LockUnlock ? 0x99 : 0x57);
	VID_WR08 (chip->PCIO, 0x3D4, 0x1F);
	VID_WR08 (chip->PCIO, 0x3D5, LockUnlock ? 0x99 : 0x57);
}

#define rivatv_lock_nv10 rivatv_lock_nv04

static ulong __init rivatv_fbsize_nv03 (struct rivatv_chip *chip)
{
	if (VID_RD32 (chip->PFB, 0) & 0x00000020) {
		if (((VID_RD32 (chip->PMC, 0) & 0xF0) == 0x20)
		    && ((VID_RD32 (chip->PMC, 0) & 0x0F) >= 0x02)) {
			/* SDRAM 128 ZX. */
			return ((1 << (VID_RD32 (chip->PFB, 0) & 0x03)) * 1024 * 1024);
		}
		else {
			return 1024 * 1024 * 8;
		}
	}
	else {
		/* SGRAM 128. */
		switch (chip->PFB[0x00000000] & 0x00000003) {
		case 0:
			return 1024 * 1024 * 8;
			break;
		case 2:
			return 1024 * 1024 * 4;
			break;
		default:
			return 1024 * 1024 * 2;
			break;
		}
	}
}

static ulong __init rivatv_fbsize_nv04 (struct rivatv_chip *chip)
{
	if (VID_RD32 (chip->PFB, 0) & 0x00000100) {
		return ((VID_RD32 (chip->PFB, 0) >> 12) & 0x0F) * 1024 * 1024 * 2
			+ 1024 * 1024 * 2;
	} else {
		switch (VID_RD32 (chip->PFB, 0) & 0x00000003) {
		case 0:
			return 1024 * 1024 * 32;
			break;
		case 1:
			return 1024 * 1024 * 4;
			break;
		case 2:
			return 1024 * 1024 * 8;
			break;
		case 3:
		default:
			return 1024 * 1024 * 16;
			break;
		}
	}
}

static ulong __init rivatv_fbsize_nv10 (struct rivatv_chip *chip)
{
	return ((VID_RD32 (chip->PFB, 0x20C) >> 20) & 0x000000FF) * 1024 * 1024;
}

/* ------------------------------------------------------------------------- *
 *
 * PCI card identification
 *
 * ------------------------------------------------------------------------- */

enum rivatv_chips {
	CH_RIVA_128 = 0,
	CH_RIVA_TNT,
	CH_RIVA_TNT2,
	CH_RIVA_UTNT2,
	CH_RIVA_VTNT2,
	CH_RIVA_UVTNT2,
	CH_RIVA_ITNT2,
	CH_GEFORCE_SDR,
	CH_GEFORCE_DDR,
	CH_QUADRO,
	CH_GEFORCE2_MX,
	CH_QUADRO2_MXR,
	CH_GEFORCE2_GTS,
	CH_GEFORCE2_ULTRA,
	CH_QUADRO2_PRO,
	CH_GEFORCE3,
	CH_GEFORCE3_1,
	CH_GEFORCE3_2,
	CH_GEFORCE4MX,
	CH_GEFORCE4,
};

/* Directly indexed by rivatv_chips, above. */
static struct rivatv_chip_info {
	const char *description;
	int arch;
	int realarch;
} rivatv_chip_info[] __devinitdata = {
	{ "RIVA-128",	    NV_ARCH_03, NV_ARCH_03 },
	{ "RIVA-TNT",	    NV_ARCH_04, NV_ARCH_04 },
	{ "RIVA-TNT2",	    NV_ARCH_04, NV_ARCH_05 },
	{ "RIVA-UTNT2",	    NV_ARCH_04, NV_ARCH_05 },
	{ "RIVA-VTNT2",	    NV_ARCH_04, NV_ARCH_05 },
	{ "RIVA-UVTNT2",    NV_ARCH_04, NV_ARCH_05 },
	{ "RIVA-ITNT2",	    NV_ARCH_04, NV_ARCH_05 },
	{ "GeForce-SDR",    NV_ARCH_10, NV_ARCH_10 },
	{ "GeForce-DDR",    NV_ARCH_10, NV_ARCH_10 },
	{ "Quadro",	    NV_ARCH_10, NV_ARCH_10 },
	{ "GeForce2-MX",    NV_ARCH_10, NV_ARCH_11 },
	{ "Quadro2-MXR",    NV_ARCH_10, NV_ARCH_11 },
	{ "GeForce2-GTS",   NV_ARCH_10, NV_ARCH_15 },
	{ "GeForce2-ULTRA", NV_ARCH_10, NV_ARCH_15 },
	{ "Quadro2-PRO",    NV_ARCH_10, NV_ARCH_15 },
	{ "Geforce3",	    NV_ARCH_20, NV_ARCH_20 },
	{ "Geforce3-TI200", NV_ARCH_20, NV_ARCH_20 },
	{ "Geforce3-TI500", NV_ARCH_20, NV_ARCH_20 },
	{ "Geforce4-MX4x0", NV_ARCH_10, NV_ARCH_17 },
	{ "Geforce4-TI4x00",NV_ARCH_20, NV_ARCH_25 },
};

/* List of supported PCI devices. */
struct pci_device_id rivatv_pci_table[] __devinitdata = {
	/* NV_ARCH_03 */
	{ PCI_VENDOR_ID_NVIDIA_SGS, PCI_DEVICE_ID_NVIDIA_SGS_RIVA128,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_RIVA_128 },
	/* NV_ARCH_04 */
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_TNT,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_RIVA_TNT },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_TNT2,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_RIVA_TNT2 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_UTNT2,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_RIVA_UTNT2 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_VTNT2,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_RIVA_VTNT2 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_UVTNT2,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_RIVA_VTNT2 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_ITNT2,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_RIVA_ITNT2 },
	/* NV_ARCH_10 */
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE_SDR,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE_SDR },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE_DDR,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE_DDR },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_QUADRO,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_QUADRO },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE2_MX,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE2_MX },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE2_MX2,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE2_MX },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_QUADRO2_MXR,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_QUADRO2_MXR },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE2_GTS,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE2_GTS },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE2_GTS2,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE2_GTS },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE2_ULTRA,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE2_ULTRA },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_QUADRO2_PRO,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_QUADRO2_PRO },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4MX460,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE4MX },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4MX440,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE4MX },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4MX420,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE4MX },
	/* NV_ARCH_20 */
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE3,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE3 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE3_1,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE3_1 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE3_2,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE3_2 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4600,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE4 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4400,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE4 },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_GEFORCE4TI4200,
	  PCI_ANY_ID, PCI_ANY_ID, 0, 0, CH_GEFORCE4 },
	{ 0, } /* end of list */
};

/* rivatv_identify_pci() tries to identify which board we're supposed to drive
   and which video decoder chip goes with it. If it cannot identify the board,
   or cannot load the decoder module, it fails. */
static int __init rivatv_identify_pci (struct pci_dev *pd, struct rivatv_info *info)
{
	int i;
	struct rivatv_card *rcard = NULL;

	/* check the list of supported cards */
	if (pd->subsystem_vendor != 0x0000) {
		for (i = 0; rivatv_cards[i].vendor != (unsigned short) PCI_ANY_ID; i++) {
			if (rivatv_cards[i].vendor == pd->vendor &&
			    rivatv_cards[i].device == pd->device &&
			    rivatv_cards[i].subsystem_vendor == pd->subsystem_vendor &&
			    rivatv_cards[i].subsystem_device == pd->subsystem_device) {
				rcard = &rivatv_cards[i];
				memcpy (&info->card, rcard, sizeof (info->card));
				memcpy (info->nicecardname, rcard->name, sizeof (info->nicecardname));
				if (rcard->audio_required != -1)
					info->audio_required = rcard->audio_required;
				if (rcard->tuner_required != -1)
					info->tuner_required = rcard->tuner_required;
				break;
			}
		}
	}

	if (rcard == NULL) {

		/* card module parameter not set */
		if (card == -1) {
			struct rivatv_vendor *vendor = NULL;

			/* check list of supported vendors */
			for (i = 0; rivatv_vendors[i].subsystem_vendor != (unsigned short) PCI_ANY_ID; i++) {
				if (rivatv_vendors[i].subsystem_vendor == pd->subsystem_vendor) {
					vendor = &rivatv_vendors[i];
					sprintf (info->nicecardname, "%s [UNKNOWN]", vendor->name);
					PRINTK_INFO ("Unknown %s board: card not in database\n", vendor->name);
					break;
				}
			}

			if (vendor == NULL) {
				strcpy (info->nicecardname, "[UNKNOWN]");
				if (pd->subsystem_vendor == 0x0000)
					PRINTK_INFO ("It is impossible to identify your board uniquely, sorry\n");
				else
					PRINTK_INFO ("Could not identify your board\n");
			}

			PRINTK_INFO ("PCI identifiers: %04X %04X %04X %04X\n",
				     pd->vendor, pd->device, pd->subsystem_vendor, pd->subsystem_device);

#ifdef CONFIG_KMOD
			if (autoload) {
				PRINTK_INFO ("Attempting to load most common decoder (SAA7108E)\n");
				return request_module ("saa7108e");
			}
#endif /* CONFIG_KMOD */
			return 0;
		}

		/* card module parameter set */
		rcard = &rivatv_cards[card - 1];
		memcpy (&info->card, rcard, sizeof (info->card));
		memcpy (info->nicecardname, rcard->name, sizeof (info->nicecardname));
		if (rcard->audio_required != -1)
			info->audio_required = rcard->audio_required;
		if (rcard->tuner_required != -1)
			info->tuner_required = rcard->tuner_required;
	}

	PRINTK_INFO ("Identified your board as %s\n", info->nicecardname);
	info->card.identified = 1;
#ifdef CONFIG_KMOD
	if (autoload) {
		if (info->tuner_required != -1) {
			PRINTK_INFO ("Attempting to load tuner module\n");
			request_module ("tuner");
		}
		if (info->audio_required != -1) {
			PRINTK_INFO ("Attempting to load tvaudio module\n");
			request_module ("tvaudio");
			PRINTK_INFO ("Attempting to load tvmixer module\n");
			request_module ("tvmixer");
		}
		PRINTK_INFO ("Attempting to load module %s\n", rcard->decoder);
		return request_module (rcard->decoder);
	}
#endif /* CONFIG_KMOD */
	return 0;
}

/* ------------------------------------------------------------------------- *
 *
 * PCI bus
 *
 * ------------------------------------------------------------------------- */

int __init rivatv_init_pci (struct pci_dev *pd,
			    const struct pci_device_id *ent)
{
	struct rivatv_info *info;
	struct rivatv_chip_info *rci = &rivatv_chip_info[ent->driver_data];
	unsigned short command_word;

	/* initialize private data */
	info = kmalloc (sizeof (struct rivatv_info), GFP_KERNEL);
	if (info == NULL)
		goto err_out;
	memset (info, 0, sizeof (struct rivatv_info));

	/* detect MMX processor extension */
	cpu_detect_mmx ();

	/* setup spin locks */
	info->video_lock = SPIN_LOCK_UNLOCKED;
	info->queue_lock = SPIN_LOCK_UNLOCKED;
	init_MUTEX (&info->decoder_lock);

	info->driver = rci->description;
	info->chip.arch = rci->arch;
	info->chip.realarch = rci->realarch;

	/* make sure this buffer is always NULL-terminated */
	info->nicecardname[RIVATV_NICENAME_BUFSIZE - 1] = '\0';

	info->tuner_required = -1;

	/* initialize some audio variables */
	info->audio_required = -1;
	info->audio.balance = info->audio.treble = info->audio.bass = 32768;
	info->audio.volume = 0xffff;

	/* check if TV-Box should be used */
	switch (tvbox) {
		/* FI1216 */
	case 1: info->tuner_required = TUNER_PHILIPS_PAL; break;
		/* FI1236 */
	case 2: info->tuner_required = TUNER_PHILIPS_NTSC; break;
		/* invalid */
	default:
		tvbox = 0; break;
	}
	if (tvbox)
		info->audio_required = 1;

	strcpy (info->nicecardname, "[UNDETERMINED]");
	if (rivatv_identify_pci (pd,info) < 0)
		PRINTK_ERR ("decoder chip module autoloading failed\n");

	/* enable PCI device and set Bus-Mastering */
	if (pci_enable_device (pd) < 0) {
		PRINTK_ERR ("failed to enable PCI device\n");
		goto err_out_kfree;
	}
	pci_read_config_word (pd, PCI_COMMAND, &command_word);
	if (!(command_word & PCI_COMMAND_MEMORY) || !(command_word & PCI_COMMAND_MASTER)) {
		PRINTK_INFO ("device not PCI enabled; enabling\n");
		command_word |= (PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER);
		pci_write_config_word (pd, PCI_COMMAND, command_word);
	}

	info->pci = pd;
	info->base0_region_size = pci_resource_len (pd, 0);
	info->base1_region_size = pci_resource_len (pd, 1);
	info->base0_region = pci_resource_start (pd, 0);
	info->base1_region = pci_resource_start (pd, 1);

	if (!request_mem_region (info->base0_region,
				 info->base0_region_size, "rivatv")) {
		PRINTK_ERR ("cannot reserve MMIO region at 0x%08lx\n",
			    info->base0_region);
		goto err_out_kfree;
	}

	if (!request_mem_region (info->base1_region,
				 info->base1_region_size, "rivatv")) {
		PRINTK_ERR ("cannot reserve FB region at 0x%08lx\n",
			    info->base1_region);
		goto err_out_free_base0;
	}

	info->control_base = ioremap_nocache (info->base0_region,
					      info->base0_region_size);
	if (!info->control_base) {
		PRINTK_ERR ("cannot ioremap MMIO base at 0x%08lx\n",
			    info->base0_region);
		goto err_out_free_base1;
	}

	info->chip.PMC =  (uint32_t *) (info->control_base + 0x00000000);
	info->chip.PFB =  (uint32_t *) (info->control_base + 0x00100000);
	info->chip.PME =  (uint32_t *) (info->control_base + 0x00000000);
	info->chip.PCIO = (uint8_t *)  (info->control_base + 0x00601000);
	info->chip.PVIO = (uint8_t *)  (info->control_base + 0x000C0000);

	/* setup chip specific functions */
	switch (info->chip.arch) {
	case NV_ARCH_03:
		info->chip.lock = rivatv_lock_nv03;
		info->chip.fbsize = rivatv_fbsize_nv03 (&info->chip);
		info->chip.PRAMIN = (uint32_t *) (info->video_base + 0x00C00000);
		break;
	case NV_ARCH_04:
		info->chip.lock = rivatv_lock_nv04;
		info->chip.fbsize = rivatv_fbsize_nv04 (&info->chip);
		info->chip.PRAMIN = (uint32_t *) (info->control_base + 0x00710000);
		info->chip.PVIDEO = (uint32_t *) (info->control_base + 0x00680000);
		break;
	case NV_ARCH_10:
	case NV_ARCH_20:
		info->chip.lock = rivatv_lock_nv10;
		info->chip.fbsize = rivatv_fbsize_nv10 (&info->chip);
		info->chip.PRAMIN = (uint32_t *) (info->control_base + 0x00710000);
		info->chip.PVIDEO = (uint32_t *) (info->control_base + 0x00000000);
		break;
	}

	switch (info->chip.arch) {
	case NV_ARCH_03:
	{
		u32 offset = PAGE_ALIGN (1024 * 768 * 4);
		info->video_base = ioremap (info->base1_region, info->base1_region_size);
		if (!info->video_base) {
			PRINTK_ERR ("cannot ioremap FB base at 0x%08lx\n",
				    info->base1_region);
			goto err_out_iounmap_ctrl;
		}
		/* This may trash your screen for resolutions greater than 1024x768, sorry. */
		info->picture_offset = offset;
		info->picture_base = (u32) info->video_base + offset;
		info->chip.PRAMIN = (uint32_t *) (info->video_base + 0x00C00000);
		break;
	}
	case NV_ARCH_04:
	case NV_ARCH_10:
	case NV_ARCH_20:
	{
		u32 size = PAGE_ALIGN (720 * 2000 * 4);
		u32 offset = info->chip.fbsize - size;
		info->picture_offset = offset;
		info->picture_base = (ulong) ioremap (info->base1_region + offset, size);
		if (!info->picture_base) {
			PRINTK_ERR ("cannot ioremap FB base at 0x%08lx\n",
				    info->base1_region);
			goto err_out_iounmap_ctrl;
		}
		break;
	}
	}

#ifdef CONFIG_MTRR
	if (mtrr) {
		info->MTRR_range = mtrr_add (info->base1_region,
					     info->base1_region_size, MTRR_TYPE_WRCOMB, 1);
		if (info->MTRR_range < 0) {
			PRINTK_ERR ("unable to setup MTRR\n");
		} else {
			info->MTRR = 1;
			PRINTK_INFO ("MTRR successfully enabled\n");
		}
	} else {
		PRINTK_INFO ("MTRR disabled\n");
	}
#endif /* CONFIG_MTRR */

	pci_set_drvdata (pd, info);
	PRINTK_INFO ("PCI nVidia NV%X card detected (%s [0x%X], %ldMB @ 0x%08lX)\n",
		     info->chip.arch, info->driver, ent->device,
		     info->chip.fbsize / 1024 / 1024, info->base1_region);
	riva_board = info;

#if 0
	rivatv_print_instances (info);
#endif
	return 0;

err_out_iounmap_ctrl:
	iounmap (info->control_base);
err_out_free_base1:
	release_mem_region (info->base1_region, info->base1_region_size);
err_out_free_base0:
	release_mem_region (info->base0_region, info->base0_region_size);
err_out_kfree:
	kfree (info);
err_out:
	return -ENODEV;
}


void __exit rivatv_cleanup_pci (struct pci_dev *pd)
{
	struct rivatv_info *info = pci_get_drvdata (pd);

	if (info == NULL)
		return;

#ifdef CONFIG_MTRR
	if (mtrr) {
		if (info->MTRR)
			mtrr_del (info->MTRR_range, info->base1_region,
				  info->base1_region_size);
	}
#endif /* CONFIG_MTRR */

	iounmap (info->control_base);
	if (info->chip.arch != NV_ARCH_03)
		iounmap ((caddr_t) info->picture_base);
	else
		iounmap (info->video_base);
	release_mem_region (info->base0_region, info->base0_region_size);
	release_mem_region (info->base1_region, info->base1_region_size);
	kfree (info);

	pci_set_drvdata (pd, NULL);
}

struct pci_driver rivatv_driver = {
	name:		"rivatv",
	id_table:	rivatv_pci_table,
	probe:		rivatv_init_pci,
	remove:		rivatv_cleanup_pci,
};
